@@ -48,8 +48,8 @@ use distribution_types::{
     ParsedUrlError, Requirement, UnresolvedRequirement, UnresolvedRequirementSpecification,
 };
 use pep508_rs::{
-    expand_env_vars, split_scheme, strip_host, Extras, Pep508Error, Pep508ErrorSource, Scheme,
-    VerbatimUrl,
+    expand_env_vars, split_scheme, strip_host, Extras, Pep508Error, Pep508ErrorSource,
+    RequirementOrigin, Scheme, VerbatimUrl,
 };
 #[cfg(feature = "http")]
 use uv_client::BaseClient;
@@ -164,9 +164,14 @@ impl FindLink {
 
 #[derive(Debug, Clone, Eq, PartialEq)]
 pub struct EditableRequirement {
+    /// The underlying [`VerbatimUrl`] from the `requirements.txt` file or similar.
     pub url: VerbatimUrl,
+    /// The extras that should be included when resolving the editable requirements.
     pub extras: Vec<ExtraName>,
+    /// The local path to the editable.
     pub path: PathBuf,
+    /// The source file containing the requirement.
+    pub origin: Option<RequirementOrigin>,
 }
 
 impl EditableRequirement {
@@ -191,6 +196,7 @@ impl EditableRequirement {
     /// We disallow URLs with schemes other than `file://` (e.g., `https://...`).
     pub fn parse(
         given: &str,
+        origin: Option<&Path>,
         working_dir: impl AsRef<Path>,
     ) -> Result<Self, RequirementsTxtParserError> {
         // Identify the extras.
@@ -265,7 +271,12 @@ impl EditableRequirement {
         // Add the verbatim representation of the URL to the `VerbatimUrl`.
         let url = url.with_given(requirement.to_string());
 
-        Ok(Self { url, extras, path })
+        Ok(Self {
+            url,
+            extras,
+            path,
+            origin: origin.map(Path::to_path_buf).map(RequirementOrigin::File),
+        })
     }
 
     /// Identify the extras in an editable URL (e.g., `../editable[dev]`).
@@ -402,12 +413,18 @@ impl RequirementsTxt {
             })?;
 
         let requirements_dir = requirements_txt.parent().unwrap_or(working_dir);
-        let data = Self::parse_inner(&content, working_dir, requirements_dir, client_builder)
-            .await
-            .map_err(|err| RequirementsTxtFileError {
-                file: requirements_txt.to_path_buf(),
-                error: err,
-            })?;
+        let data = Self::parse_inner(
+            &content,
+            working_dir,
+            requirements_dir,
+            client_builder,
+            requirements_txt,
+        )
+        .await
+        .map_err(|err| RequirementsTxtFileError {
+            file: requirements_txt.to_path_buf(),
+            error: err,
+        })?;
         if data == Self::default() {
             warn_user!(
                 "Requirements file {} does not contain any dependencies",
@@ -429,11 +446,12 @@ impl RequirementsTxt {
         working_dir: &Path,
         requirements_dir: &Path,
         client_builder: &BaseClientBuilder<'_>,
+        requirements_txt: &Path,
     ) -> Result<Self, RequirementsTxtParserError> {
         let mut s = Scanner::new(content);
 
         let mut data = Self::default();
-        while let Some(statement) = parse_entry(&mut s, content, working_dir)? {
+        while let Some(statement) = parse_entry(&mut s, content, working_dir, requirements_txt)? {
             match statement {
                 RequirementsTxtStatement::Requirements {
                     filename,
@@ -511,7 +529,9 @@ impl RequirementsTxt {
                             }
                         }
                     }
-                    data.constraints.extend(sub_constraints.constraints);
+                    for constraint in sub_constraints.constraints {
+                        data.constraints.push(constraint);
+                    }
                 }
                 RequirementsTxtStatement::RequirementEntry(requirement_entry) => {
                     data.requirements.push(requirement_entry);
@@ -585,6 +605,7 @@ fn parse_entry(
     s: &mut Scanner,
     content: &str,
     working_dir: &Path,
+    requirements_txt: &Path,
 ) -> Result<Option<RequirementsTxtStatement>, RequirementsTxtParserError> {
     // Eat all preceding whitespace, this may run us to the end of file
     eat_wrappable_whitespace(s);
@@ -613,8 +634,9 @@ fn parse_entry(
         }
     } else if s.eat_if("-e") || s.eat_if("--editable") {
         let path_or_url = parse_value(content, s, |c: char| !['\n', '\r', '#'].contains(&c))?;
-        let editable_requirement = EditableRequirement::parse(path_or_url, working_dir)
-            .map_err(|err| err.with_offset(start))?;
+        let editable_requirement =
+            EditableRequirement::parse(path_or_url, Some(requirements_txt), working_dir)
+                .map_err(|err| err.with_offset(start))?;
         RequirementsTxtStatement::EditableRequirement(editable_requirement)
     } else if s.eat_if("-i") || s.eat_if("--index-url") {
         let given = parse_value(content, s, |c: char| !['\n', '\r', '#'].contains(&c))?;
@@ -676,7 +698,13 @@ fn parse_entry(
         })?;
         RequirementsTxtStatement::OnlyBinary(NoBuild::from_arg(specifier))
     } else if s.at(char::is_ascii_alphanumeric) || s.at(|char| matches!(char, '.' | '/' | '$')) {
-        let (requirement, hashes) = parse_requirement_and_hashes(s, content, working_dir)?;
+        let source = if requirements_txt == Path::new("-") {
+            None
+        } else {
+            Some(requirements_txt)
+        };
+
+        let (requirement, hashes) = parse_requirement_and_hashes(s, content, source, working_dir)?;
         RequirementsTxtStatement::RequirementEntry(RequirementEntry {
             requirement,
             hashes,
@@ -738,6 +766,7 @@ fn eat_trailing_line(content: &str, s: &mut Scanner) -> Result<(), RequirementsT
 fn parse_requirement_and_hashes(
     s: &mut Scanner,
     content: &str,
+    source: Option<&Path>,
     working_dir: &Path,
 ) -> Result<(RequirementsTxtRequirement, Vec<String>), RequirementsTxtParserError> {
     // PEP 508 requirement
@@ -795,8 +824,15 @@ fn parse_requirement_and_hashes(
         }
     }
 
-    let requirement =
-        RequirementsTxtRequirement::parse(requirement, working_dir).map_err(|err| match err {
+    let requirement = RequirementsTxtRequirement::parse(requirement, working_dir)
+        .map(|requirement| {
+            if let Some(source) = source {
+                requirement.with_origin(RequirementOrigin::File(source.to_path_buf()))
+            } else {
+                requirement
+            }
+        })
+        .map_err(|err| match err {
             RequirementsTxtRequirementError::ParsedUrl(err) => {
                 RequirementsTxtParserError::ParsedUrl {
                     source: err,
@@ -1332,7 +1368,21 @@ mod test {
     use crate::{calculate_row_column, EditableRequirement, RequirementsTxt};
 
     fn workspace_test_data_dir() -> PathBuf {
-        PathBuf::from("./test-data").canonicalize().unwrap()
+        Path::new("./test-data").simple_canonicalize().unwrap()
+    }
+
+    /// Filter a path for use in snapshots; in particular, match the Windows debug representation
+    /// of a path.
+    ///
+    /// We replace backslashes to match the debug representation for paths, and match _either_
+    /// backslashes or forward slashes as the latter appear when constructing a path from a URL.
+    fn path_filter(path: &Path) -> String {
+        regex::escape(&path.simplified_display().to_string()).replace(r"\\", r"(\\\\|/)")
+    }
+
+    /// Return the insta filters for a given path.
+    fn path_filters(filter: &str) -> Vec<(&str, &str)> {
+        vec![(filter, "<REQUIREMENTS_DIR>"), (r"\\\\", "/")]
     }
 
     #[test_case(Path::new("basic.txt"))]
@@ -1350,13 +1400,21 @@ mod test {
         let working_dir = workspace_test_data_dir().join("requirements-txt");
         let requirements_txt = working_dir.join(path);
 
-        let actual =
-            RequirementsTxt::parse(requirements_txt, &working_dir, &BaseClientBuilder::new())
-                .await
-                .unwrap();
+        let actual = RequirementsTxt::parse(
+            requirements_txt.clone(),
+            &working_dir,
+            &BaseClientBuilder::new(),
+        )
+        .await
+        .unwrap();
 
         let snapshot = format!("parse-{}", path.to_string_lossy());
-        insta::assert_debug_snapshot!(snapshot, actual);
+
+        insta::with_settings!({
+            filters => path_filters(&path_filter(&working_dir)),
+        }, {
+            insta::assert_debug_snapshot!(snapshot, actual);
+        });
     }
 
     #[test_case(Path::new("basic.txt"))]
@@ -1401,7 +1459,12 @@ mod test {
                 .unwrap();
 
         let snapshot = format!("line-endings-{}", path.to_string_lossy());
-        insta::assert_debug_snapshot!(snapshot, actual);
+
+        insta::with_settings!({
+            filters => path_filters(&path_filter(temp_dir.path())),
+        }, {
+            insta::assert_debug_snapshot!(snapshot, actual);
+        });
     }
 
     #[cfg(unix)]
@@ -1417,10 +1480,9 @@ mod test {
                 .unwrap();
 
         let snapshot = format!("parse-unix-{}", path.to_string_lossy());
-        let pattern = regex::escape(&working_dir.simplified_display().to_string());
-        let filters = vec![(pattern.as_str(), "[WORKSPACE_DIR]")];
+
         insta::with_settings!({
-            filters => filters
+            filters => path_filters(&path_filter(&working_dir)),
         }, {
             insta::assert_debug_snapshot!(snapshot, actual);
         });
@@ -1439,15 +1501,9 @@ mod test {
                 .unwrap();
 
         let snapshot = format!("parse-windows-{}", path.to_string_lossy());
-        let pattern = regex::escape(
-            &working_dir
-                .simplified_display()
-                .to_string()
-                .replace('\\', "/"),
-        );
-        let filters = vec![(pattern.as_str(), "[WORKSPACE_DIR]")];
+
         insta::with_settings!({
-            filters => filters
+            filters => path_filters(&path_filter(&working_dir)),
         }, {
             insta::assert_debug_snapshot!(snapshot, actual);
         });
@@ -1480,7 +1536,6 @@ mod test {
         let filters = vec![
             (requirement_txt.as_str(), "<REQUIREMENTS_TXT>"),
             (missing_txt.as_str(), "<MISSING_TXT>"),
-            (r"\\", "/"),
         ];
         insta::with_settings!({
             filters => filters,
@@ -1512,10 +1567,7 @@ mod test {
         let errors = anyhow::Error::new(error).chain().join("\n");
 
         let requirement_txt = regex::escape(&requirements_txt.path().user_display().to_string());
-        let filters = vec![
-            (requirement_txt.as_str(), "<REQUIREMENTS_TXT>"),
-            (r"\\", "/"),
-        ];
+        let filters = vec![(requirement_txt.as_str(), "<REQUIREMENTS_TXT>")];
         insta::with_settings!({
             filters => filters
         }, {
@@ -1548,10 +1600,7 @@ mod test {
         let errors = anyhow::Error::new(error).chain().join("\n");
 
         let requirement_txt = regex::escape(&requirements_txt.path().user_display().to_string());
-        let filters = vec![
-            (requirement_txt.as_str(), "<REQUIREMENTS_TXT>"),
-            (r"\\", "/"),
-        ];
+        let filters = vec![(requirement_txt.as_str(), "<REQUIREMENTS_TXT>")];
         insta::with_settings!({
             filters => filters
         }, {
@@ -1584,10 +1633,7 @@ mod test {
         let errors = anyhow::Error::new(error).chain().join("\n");
 
         let requirement_txt = regex::escape(&requirements_txt.path().user_display().to_string());
-        let filters = vec![
-            (requirement_txt.as_str(), "<REQUIREMENTS_TXT>"),
-            (r"\\", "/"),
-        ];
+        let filters = vec![(requirement_txt.as_str(), "<REQUIREMENTS_TXT>")];
         insta::with_settings!({
             filters => filters
         }, {
@@ -1648,10 +1694,7 @@ mod test {
         let errors = anyhow::Error::new(error).chain().join("\n");
 
         let requirement_txt = regex::escape(&requirements_txt.path().user_display().to_string());
-        let filters = vec![
-            (requirement_txt.as_str(), "<REQUIREMENTS_TXT>"),
-            (r"\\", "/"),
-        ];
+        let filters = vec![(requirement_txt.as_str(), "<REQUIREMENTS_TXT>")];
         insta::with_settings!({
             filters => filters
         }, {
@@ -1682,10 +1725,7 @@ mod test {
         let errors = anyhow::Error::new(error).chain().join("\n");
 
         let requirement_txt = regex::escape(&requirements_txt.path().user_display().to_string());
-        let filters = vec![
-            (requirement_txt.as_str(), "<REQUIREMENTS_TXT>"),
-            (r"\\", "/"),
-        ];
+        let filters = vec![(requirement_txt.as_str(), "<REQUIREMENTS_TXT>")];
         insta::with_settings!({
             filters => filters
         }, {
@@ -1721,10 +1761,7 @@ mod test {
         let errors = anyhow::Error::new(error).chain().join("\n");
 
         let requirement_txt = regex::escape(&requirements_txt.path().user_display().to_string());
-        let filters = vec![
-            (requirement_txt.as_str(), "<REQUIREMENTS_TXT>"),
-            (r"\\", "/"),
-        ];
+        let filters = vec![(requirement_txt.as_str(), "<REQUIREMENTS_TXT>")];
         insta::with_settings!({
             filters => filters
         }, {
@@ -1764,33 +1801,43 @@ mod test {
         )
         .await
         .unwrap();
-        insta::assert_debug_snapshot!(requirements, @r###"
-        RequirementsTxt {
-            requirements: [
-                RequirementEntry {
-                    requirement: Named(
-                        Requirement {
-                            name: PackageName(
-                                "flask",
-                            ),
-                            extras: [],
-                            version_or_url: None,
-                            marker: None,
-                        },
-                    ),
-                    hashes: [],
-                },
-            ],
-            constraints: [],
-            editables: [],
-            index_url: None,
-            extra_index_urls: [],
-            find_links: [],
-            no_index: false,
-            no_binary: None,
-            only_binary: None,
-        }
-        "###);
+
+        insta::with_settings!({
+            filters => path_filters(&path_filter(temp_dir.path())),
+        }, {
+            insta::assert_debug_snapshot!(requirements, @r###"
+            RequirementsTxt {
+                requirements: [
+                    RequirementEntry {
+                        requirement: Named(
+                            Requirement {
+                                name: PackageName(
+                                    "flask",
+                                ),
+                                extras: [],
+                                version_or_url: None,
+                                marker: None,
+                                origin: Some(
+                                    File(
+                                        "<REQUIREMENTS_DIR>/subdir/sibling.txt",
+                                    ),
+                                ),
+                            },
+                        ),
+                        hashes: [],
+                    },
+                ],
+                constraints: [],
+                editables: [],
+                index_url: None,
+                extra_index_urls: [],
+                find_links: [],
+                no_index: false,
+                no_binary: None,
+                only_binary: None,
+            }
+            "###);
+        });
 
         Ok(())
     }
@@ -1818,39 +1865,49 @@ mod test {
         )
         .await
         .unwrap();
-        insta::assert_debug_snapshot!(requirements, @r###"
-        RequirementsTxt {
-            requirements: [
-                RequirementEntry {
-                    requirement: Named(
-                        Requirement {
-                            name: PackageName(
-                                "flask",
-                            ),
-                            extras: [],
-                            version_or_url: None,
-                            marker: None,
-                        },
-                    ),
-                    hashes: [],
-                },
-            ],
-            constraints: [],
-            editables: [],
-            index_url: None,
-            extra_index_urls: [],
-            find_links: [],
-            no_index: false,
-            no_binary: Packages(
-                [
-                    PackageName(
-                        "flask",
-                    ),
+
+        insta::with_settings!({
+            filters => path_filters(&path_filter(temp_dir.path())),
+        }, {
+            insta::assert_debug_snapshot!(requirements, @r###"
+            RequirementsTxt {
+                requirements: [
+                    RequirementEntry {
+                        requirement: Named(
+                            Requirement {
+                                name: PackageName(
+                                    "flask",
+                                ),
+                                extras: [],
+                                version_or_url: None,
+                                marker: None,
+                                origin: Some(
+                                    File(
+                                        "<REQUIREMENTS_DIR>/requirements.txt",
+                                    ),
+                                ),
+                            },
+                        ),
+                        hashes: [],
+                    },
                 ],
-            ),
-            only_binary: None,
-        }
-        "###);
+                constraints: [],
+                editables: [],
+                index_url: None,
+                extra_index_urls: [],
+                find_links: [],
+                no_index: false,
+                no_binary: Packages(
+                    [
+                        PackageName(
+                            "flask",
+                        ),
+                    ],
+                ),
+                only_binary: None,
+            }
+            "###);
+        });
 
         Ok(())
     }
@@ -1884,40 +1941,49 @@ mod test {
         .await
         .unwrap();
 
-        insta::assert_debug_snapshot!(requirements, @r###"
-        RequirementsTxt {
-            requirements: [],
-            constraints: [],
-            editables: [
-                EditableRequirement {
-                    url: VerbatimUrl {
-                        url: Url {
-                            scheme: "file",
-                            cannot_be_a_base: false,
-                            username: "",
-                            password: None,
-                            host: None,
-                            port: None,
-                            path: "/foo/bar",
-                            query: None,
-                            fragment: None,
+        insta::with_settings!({
+            filters => path_filters(&path_filter(temp_dir.path())),
+        }, {
+            insta::assert_debug_snapshot!(requirements, @r###"
+            RequirementsTxt {
+                requirements: [],
+                constraints: [],
+                editables: [
+                    EditableRequirement {
+                        url: VerbatimUrl {
+                            url: Url {
+                                scheme: "file",
+                                cannot_be_a_base: false,
+                                username: "",
+                                password: None,
+                                host: None,
+                                port: None,
+                                path: "/foo/bar",
+                                query: None,
+                                fragment: None,
+                            },
+                            given: Some(
+                                "/foo/bar",
+                            ),
                         },
-                        given: Some(
-                            "/foo/bar",
+                        extras: [],
+                        path: "/foo/bar",
+                        origin: Some(
+                            File(
+                                "<REQUIREMENTS_DIR>/grandchild.txt",
+                            ),
                         ),
                     },
-                    extras: [],
-                    path: "/foo/bar",
-                },
-            ],
-            index_url: None,
-            extra_index_urls: [],
-            find_links: [],
-            no_index: true,
-            no_binary: None,
-            only_binary: None,
-        }
-        "###);
+                ],
+                index_url: None,
+                extra_index_urls: [],
+                find_links: [],
+                no_index: true,
+                no_binary: None,
+                only_binary: None,
+            }
+            "###);
+        });
 
         Ok(())
     }
@@ -1952,10 +2018,7 @@ mod test {
         let errors = anyhow::Error::new(error).chain().join("\n");
 
         let requirement_txt = regex::escape(&requirements_txt.path().user_display().to_string());
-        let filters = vec![
-            (requirement_txt.as_str(), "<REQUIREMENTS_TXT>"),
-            (r"\\", "/"),
-        ];
+        let filters = vec![(requirement_txt.as_str(), "<REQUIREMENTS_TXT>")];
         insta::with_settings!({
             filters => filters
         }, {
@@ -2000,154 +2063,184 @@ mod test {
         )
         .await
         .unwrap();
-        insta::assert_debug_snapshot!(requirements, @r###"
-        RequirementsTxt {
-            requirements: [
-                RequirementEntry {
-                    requirement: Named(
-                        Requirement {
-                            name: PackageName(
-                                "httpx",
-                            ),
-                            extras: [],
-                            version_or_url: None,
-                            marker: None,
-                        },
-                    ),
-                    hashes: [],
-                },
-                RequirementEntry {
-                    requirement: Named(
-                        Requirement {
-                            name: PackageName(
-                                "flask",
-                            ),
-                            extras: [],
-                            version_or_url: Some(
-                                VersionSpecifier(
-                                    VersionSpecifiers(
-                                        [
-                                            VersionSpecifier {
-                                                operator: Equal,
-                                                version: "3.0.0",
-                                            },
-                                        ],
+
+        insta::with_settings!({
+            filters => path_filters(&path_filter(temp_dir.path())),
+        }, {
+            insta::assert_debug_snapshot!(requirements, @r###"
+            RequirementsTxt {
+                requirements: [
+                    RequirementEntry {
+                        requirement: Named(
+                            Requirement {
+                                name: PackageName(
+                                    "httpx",
+                                ),
+                                extras: [],
+                                version_or_url: None,
+                                marker: None,
+                                origin: Some(
+                                    File(
+                                        "<REQUIREMENTS_DIR>/./sibling.txt",
                                     ),
                                 ),
-                            ),
-                            marker: None,
-                        },
-                    ),
-                    hashes: [
-                        "sha256:1234567890abcdef1234567890abcdef1234567890abcdef1234567890abcdef",
-                    ],
-                },
-                RequirementEntry {
-                    requirement: Named(
-                        Requirement {
-                            name: PackageName(
-                                "requests",
-                            ),
-                            extras: [],
-                            version_or_url: Some(
-                                VersionSpecifier(
-                                    VersionSpecifiers(
-                                        [
-                                            VersionSpecifier {
-                                                operator: Equal,
-                                                version: "2.26.0",
-                                            },
-                                        ],
-                                    ),
-                                ),
-                            ),
-                            marker: None,
-                        },
-                    ),
-                    hashes: [
-                        "sha256:fedcba0987654321fedcba0987654321fedcba0987654321fedcba0987654321",
-                    ],
-                },
-                RequirementEntry {
-                    requirement: Named(
-                        Requirement {
-                            name: PackageName(
-                                "black",
-                            ),
-                            extras: [],
-                            version_or_url: Some(
-                                VersionSpecifier(
-                                    VersionSpecifiers(
-                                        [
-                                            VersionSpecifier {
-                                                operator: Equal,
-                                                version: "21.12b0",
-                                            },
-                                        ],
-                                    ),
-                                ),
-                            ),
-                            marker: None,
-                        },
-                    ),
-                    hashes: [],
-                },
-                RequirementEntry {
-                    requirement: Named(
-                        Requirement {
-                            name: PackageName(
-                                "mypy",
-                            ),
-                            extras: [],
-                            version_or_url: Some(
-                                VersionSpecifier(
-                                    VersionSpecifiers(
-                                        [
-                                            VersionSpecifier {
-                                                operator: Equal,
-                                                version: "0.910",
-                                            },
-                                        ],
-                                    ),
-                                ),
-                            ),
-                            marker: None,
-                        },
-                    ),
-                    hashes: [],
-                },
-            ],
-            constraints: [],
-            editables: [],
-            index_url: Some(
-                VerbatimUrl {
-                    url: Url {
-                        scheme: "https",
-                        cannot_be_a_base: false,
-                        username: "",
-                        password: None,
-                        host: Some(
-                            Domain(
-                                "test.pypi.org",
-                            ),
+                            },
                         ),
-                        port: None,
-                        path: "/simple/",
-                        query: None,
-                        fragment: None,
+                        hashes: [],
                     },
-                    given: Some(
-                        "https://test.pypi.org/simple/",
-                    ),
-                },
-            ),
-            extra_index_urls: [],
-            find_links: [],
-            no_index: false,
-            no_binary: All,
-            only_binary: None,
-        }
-        "###);
+                    RequirementEntry {
+                        requirement: Named(
+                            Requirement {
+                                name: PackageName(
+                                    "flask",
+                                ),
+                                extras: [],
+                                version_or_url: Some(
+                                    VersionSpecifier(
+                                        VersionSpecifiers(
+                                            [
+                                                VersionSpecifier {
+                                                    operator: Equal,
+                                                    version: "3.0.0",
+                                                },
+                                            ],
+                                        ),
+                                    ),
+                                ),
+                                marker: None,
+                                origin: Some(
+                                    File(
+                                        "<REQUIREMENTS_DIR>/requirements.txt",
+                                    ),
+                                ),
+                            },
+                        ),
+                        hashes: [
+                            "sha256:1234567890abcdef1234567890abcdef1234567890abcdef1234567890abcdef",
+                        ],
+                    },
+                    RequirementEntry {
+                        requirement: Named(
+                            Requirement {
+                                name: PackageName(
+                                    "requests",
+                                ),
+                                extras: [],
+                                version_or_url: Some(
+                                    VersionSpecifier(
+                                        VersionSpecifiers(
+                                            [
+                                                VersionSpecifier {
+                                                    operator: Equal,
+                                                    version: "2.26.0",
+                                                },
+                                            ],
+                                        ),
+                                    ),
+                                ),
+                                marker: None,
+                                origin: Some(
+                                    File(
+                                        "<REQUIREMENTS_DIR>/requirements.txt",
+                                    ),
+                                ),
+                            },
+                        ),
+                        hashes: [
+                            "sha256:fedcba0987654321fedcba0987654321fedcba0987654321fedcba0987654321",
+                        ],
+                    },
+                    RequirementEntry {
+                        requirement: Named(
+                            Requirement {
+                                name: PackageName(
+                                    "black",
+                                ),
+                                extras: [],
+                                version_or_url: Some(
+                                    VersionSpecifier(
+                                        VersionSpecifiers(
+                                            [
+                                                VersionSpecifier {
+                                                    operator: Equal,
+                                                    version: "21.12b0",
+                                                },
+                                            ],
+                                        ),
+                                    ),
+                                ),
+                                marker: None,
+                                origin: Some(
+                                    File(
+                                        "<REQUIREMENTS_DIR>/requirements.txt",
+                                    ),
+                                ),
+                            },
+                        ),
+                        hashes: [],
+                    },
+                    RequirementEntry {
+                        requirement: Named(
+                            Requirement {
+                                name: PackageName(
+                                    "mypy",
+                                ),
+                                extras: [],
+                                version_or_url: Some(
+                                    VersionSpecifier(
+                                        VersionSpecifiers(
+                                            [
+                                                VersionSpecifier {
+                                                    operator: Equal,
+                                                    version: "0.910",
+                                                },
+                                            ],
+                                        ),
+                                    ),
+                                ),
+                                marker: None,
+                                origin: Some(
+                                    File(
+                                        "<REQUIREMENTS_DIR>/requirements.txt",
+                                    ),
+                                ),
+                            },
+                        ),
+                        hashes: [],
+                    },
+                ],
+                constraints: [],
+                editables: [],
+                index_url: Some(
+                    VerbatimUrl {
+                        url: Url {
+                            scheme: "https",
+                            cannot_be_a_base: false,
+                            username: "",
+                            password: None,
+                            host: Some(
+                                Domain(
+                                    "test.pypi.org",
+                                ),
+                            ),
+                            port: None,
+                            path: "/simple/",
+                            query: None,
+                            fragment: None,
+                        },
+                        given: Some(
+                            "https://test.pypi.org/simple/",
+                        ),
+                    },
+                ),
+                extra_index_urls: [],
+                find_links: [],
+                no_index: false,
+                no_binary: All,
+                only_binary: None,
+            }
+            "###);
+        });
 
         Ok(())
     }
@@ -2192,10 +2285,7 @@ mod test {
         let errors = anyhow::Error::new(error).chain().join("\n");
 
         let requirement_txt = regex::escape(&requirements_txt.path().user_display().to_string());
-        let filters = vec![
-            (requirement_txt.as_str(), "<REQUIREMENTS_TXT>"),
-            (r"\\", "/"),
-        ];
+        let filters = vec![(requirement_txt.as_str(), "<REQUIREMENTS_TXT>")];
         insta::with_settings!({
             filters => filters
         }, {
