@@ -3,7 +3,7 @@ use std::path::Path;
 use thiserror::Error;
 
 use distribution_types::ParsedUrlError;
-use pep508_rs::{Pep508Error, Pep508ErrorSource, UnnamedRequirement};
+use pep508_rs::{Pep508Error, Pep508ErrorSource, RequirementOrigin, UnnamedRequirement};
 
 /// A requirement specifier in a `requirements.txt` file.
 ///
@@ -18,6 +18,17 @@ pub enum RequirementsTxtRequirement {
     Unnamed(UnnamedRequirement),
 }
 
+impl RequirementsTxtRequirement {
+    /// Set the source file containing the requirement.
+    #[must_use]
+    pub fn with_origin(self, origin: RequirementOrigin) -> Self {
+        match self {
+            Self::Named(requirement) => Self::Named(requirement.with_origin(origin)),
+            Self::Unnamed(requirement) => Self::Unnamed(requirement.with_origin(origin)),
+        }
+    }
+}
+
 #[derive(Debug, Error)]
 pub enum RequirementsTxtRequirementError {
     #[error(transparent)]
