@@ -2,7 +2,7 @@ mod cache;
 mod credentials;
 mod keyring;
 mod middleware;
-mod netloc;
+mod realm;
 
 use std::sync::Arc;
 
@@ -11,8 +11,9 @@ use credentials::Credentials;
 
 pub use keyring::KeyringProvider;
 pub use middleware::AuthMiddleware;
-use netloc::NetLoc;
 use once_cell::sync::Lazy;
+use realm::Realm;
+use tracing::trace;
 use url::Url;
 
 // TODO(zanieb): Consider passing a cache explicitly throughout
@@ -27,6 +28,7 @@ pub(crate) static CREDENTIALS_CACHE: Lazy<CredentialsCache> = Lazy::new(Credenti
 /// Returns `true` if the store was updated.
 pub fn store_credentials_from_url(url: &Url) -> bool {
     if let Some(credentials) = Credentials::from_url(url) {
+        trace!("Caching credentials for {url}");
         CREDENTIALS_CACHE.insert(url, Arc::new(credentials));
         true
     } else {
