@@ -11,9 +11,10 @@ use rustc_hash::FxHashSet;
 use tempfile::{tempdir, TempDir};
 use tracing::debug;
 
+pub use archive::ArchiveId;
 use distribution_types::InstalledDist;
 use pypi_types::Metadata23;
-use uv_fs::directories;
+use uv_fs::{cachedir, directories};
 use uv_normalize::PackageName;
 
 pub use crate::by_timestamp::CachedByTimestamp;
@@ -23,7 +24,6 @@ use crate::removal::{rm_rf, Removal};
 pub use crate::timestamp::Timestamp;
 pub use crate::wheel::WheelCache;
 use crate::wheel::WheelCacheKind;
-pub use archive::ArchiveId;
 
 mod archive;
 mod by_timestamp;
@@ -376,7 +376,7 @@ impl Cache {
 pub enum CacheBucket {
     /// Wheels (excluding built wheels), alongside their metadata and cache policy.
     ///
-    /// There are three kinds from cache entries: Wheel metadata and policy as MsgPack files, the
+    /// There are three kinds from cache entries: Wheel metadata and policy as `MsgPack` files, the
     /// wheels themselves, and the unzipped wheel archives. If a wheel file is over an in-memory
     /// size threshold, we first download the zip file into the cache, then unzip it into a
     /// directory with the same name (exclusive of the `.whl` extension).
@@ -559,7 +559,7 @@ pub enum CacheBucket {
     ///
     /// # Example
     ///
-    /// The contents of each of the MsgPack files has a timestamp field in unix time, the [PEP 508]
+    /// The contents of each of the `MsgPack` files has a timestamp field in unix time, the [PEP 508]
     /// markers and some information from the `sys`/`sysconfig` modules.
     ///
     /// ```json
@@ -770,41 +770,7 @@ impl ArchiveTimestamp {
         if metadata.is_file() {
             Ok(Some(Self::Exact(Timestamp::from_metadata(&metadata))))
         } else {
-            // Compute the modification timestamp for the `pyproject.toml`, `setup.py`, and
-            // `setup.cfg` files, if they exist.
-            let pyproject_toml = path
-                .as_ref()
-                .join("pyproject.toml")
-                .metadata()
-                .ok()
-                .filter(std::fs::Metadata::is_file)
-                .as_ref()
-                .map(Timestamp::from_metadata);
-
-            let setup_py = path
-                .as_ref()
-                .join("setup.py")
-                .metadata()
-                .ok()
-                .filter(std::fs::Metadata::is_file)
-                .as_ref()
-                .map(Timestamp::from_metadata);
-
-            let setup_cfg = path
-                .as_ref()
-                .join("setup.cfg")
-                .metadata()
-                .ok()
-                .filter(std::fs::Metadata::is_file)
-                .as_ref()
-                .map(Timestamp::from_metadata);
-
-            // Take the most recent timestamp of the three files.
-            let Some(timestamp) = max(pyproject_toml, max(setup_py, setup_cfg)) else {
-                return Ok(None);
-            };
-
-            Ok(Some(Self::Approximate(timestamp)))
+            Self::from_source_tree(path)
         }
     }
 
@@ -814,6 +780,48 @@ impl ArchiveTimestamp {
         Ok(Self::Exact(Timestamp::from_metadata(&metadata)))
     }
 
+    /// Return the modification timestamp for a source tree, i.e., a directory.
+    ///
+    /// If the source tree doesn't contain an entrypoint (i.e., no `pyproject.toml`, `setup.py`, or
+    /// `setup.cfg`), returns `None`.
+    pub fn from_source_tree(path: impl AsRef<Path>) -> Result<Option<Self>, io::Error> {
+        // Compute the modification timestamp for the `pyproject.toml`, `setup.py`, and
+        // `setup.cfg` files, if they exist.
+        let pyproject_toml = path
+            .as_ref()
+            .join("pyproject.toml")
+            .metadata()
+            .ok()
+            .filter(std::fs::Metadata::is_file)
+            .as_ref()
+            .map(Timestamp::from_metadata);
+
+        let setup_py = path
+            .as_ref()
+            .join("setup.py")
+            .metadata()
+            .ok()
+            .filter(std::fs::Metadata::is_file)
+            .as_ref()
+            .map(Timestamp::from_metadata);
+
+        let setup_cfg = path
+            .as_ref()
+            .join("setup.cfg")
+            .metadata()
+            .ok()
+            .filter(std::fs::Metadata::is_file)
+            .as_ref()
+            .map(Timestamp::from_metadata);
+
+        // Take the most recent timestamp of the three files.
+        let Some(timestamp) = max(pyproject_toml, max(setup_py, setup_cfg)) else {
+            return Ok(None);
+        };
+
+        Ok(Some(Self::Approximate(timestamp)))
+    }
+
     /// Return the modification timestamp for an archive.
     pub fn timestamp(&self) -> Timestamp {
         match self {
@@ -898,13 +906,17 @@ pub enum Refresh {
 
 impl Refresh {
     /// Determine the refresh strategy to use based on the command-line arguments.
-    pub fn from_args(refresh: bool, refresh_package: Vec<PackageName>) -> Self {
-        if refresh {
-            Self::All(Timestamp::now())
-        } else if !refresh_package.is_empty() {
-            Self::Packages(refresh_package, Timestamp::now())
-        } else {
-            Self::None
+    pub fn from_args(refresh: Option<bool>, refresh_package: Vec<PackageName>) -> Self {
+        match refresh {
+            Some(true) => Self::All(Timestamp::now()),
+            Some(false) => Self::None,
+            None => {
+                if refresh_package.is_empty() {
+                    Self::None
+                } else {
+                    Self::Packages(refresh_package, Timestamp::now())
+                }
+            }
         }
     }
 
