@@ -102,7 +102,7 @@ impl<'a> RegistryClientBuilder<'a> {
     }
 
     #[must_use]
-    pub fn cache<T>(mut self, cache: Cache) -> Self {
+    pub fn cache(mut self, cache: Cache) -> Self {
         self.cache = cache;
         self
     }
@@ -221,7 +221,7 @@ impl RegistryClient {
     ) -> Result<Vec<(IndexUrl, OwnedArchive<SimpleMetadata>)>, Error> {
         let mut it = self.index_urls.indexes().peekable();
         if it.peek().is_none() {
-            return Err(ErrorKind::NoIndex(package_name.as_ref().to_string()).into());
+            return Err(ErrorKind::NoIndex(package_name.to_string()).into());
         }
 
         let mut results = Vec::new();
@@ -239,6 +239,7 @@ impl RegistryClient {
                     ErrorKind::Offline(_) => continue,
                     ErrorKind::ReqwestError(err) => {
                         if err.status() == Some(StatusCode::NOT_FOUND)
+                            || err.status() == Some(StatusCode::UNAUTHORIZED)
                             || err.status() == Some(StatusCode::FORBIDDEN)
                         {
                             continue;
@@ -344,7 +345,7 @@ impl RegistryClient {
                 };
                 OwnedArchive::from_unarchived(&unarchived)
             }
-            .boxed()
+            .boxed_local()
             .instrument(info_span!("parse_simple_api", package = %package_name))
         };
         let result = self
@@ -518,6 +519,7 @@ impl RegistryClient {
                 let mut reader = AsyncHttpRangeReader::from_head_response(
                     self.uncached_client().client(),
                     response,
+                    url.clone(),
                     headers,
                 )
                 .await
@@ -533,7 +535,7 @@ impl RegistryClient {
                 })?;
                 Ok::<Metadata23, CachedClientError<Error>>(metadata)
             }
-            .boxed()
+            .boxed_local()
             .instrument(info_span!("read_metadata_range_request", wheel = %filename))
         };
 
