@@ -12,15 +12,16 @@ use itertools::Itertools;
 use rustc_hash::FxHashMap;
 use tracing::{debug, instrument};
 
-use distribution_types::{IndexLocations, Name, Resolution, SourceDist};
-use pep508_rs::Requirement;
+use distribution_types::{IndexLocations, Name, Requirement, Resolution, SourceDist};
 use uv_build::{SourceBuild, SourceBuildContext};
 use uv_cache::Cache;
 use uv_client::RegistryClient;
+use uv_configuration::Concurrency;
 use uv_configuration::{BuildKind, ConfigSettings, NoBinary, NoBuild, Reinstall, SetupPyStrategy};
+use uv_distribution::DistributionDatabase;
 use uv_installer::{Downloader, Installer, Plan, Planner, SitePackages};
 use uv_interpreter::{Interpreter, PythonEnvironment};
-use uv_resolver::{FlatIndex, InMemoryIndex, Manifest, Options, Resolver};
+use uv_resolver::{FlatIndex, InMemoryIndex, Manifest, Options, PythonRequirement, Resolver};
 use uv_types::{BuildContext, BuildIsolation, EmptyInstalledPackages, HashStrategy, InFlight};
 
 /// The main implementation of [`BuildContext`], used by the CLI, see [`BuildContext`]
@@ -42,6 +43,7 @@ pub struct BuildDispatch<'a> {
     source_build_context: SourceBuildContext,
     options: Options,
     build_extra_env_vars: FxHashMap<OsString, OsString>,
+    concurrency: Concurrency,
 }
 
 impl<'a> BuildDispatch<'a> {
@@ -60,6 +62,7 @@ impl<'a> BuildDispatch<'a> {
         link_mode: install_wheel_rs::linker::LinkMode,
         no_build: &'a NoBuild,
         no_binary: &'a NoBinary,
+        concurrency: Concurrency,
     ) -> Self {
         Self {
             client,
@@ -75,6 +78,7 @@ impl<'a> BuildDispatch<'a> {
             link_mode,
             no_build,
             no_binary,
+            concurrency,
             source_build_context: SourceBuildContext::default(),
             options: Options::default(),
             build_extra_env_vars: FxHashMap::default(),
@@ -136,19 +140,21 @@ impl<'a> BuildContext for BuildDispatch<'a> {
 
     async fn resolve<'data>(&'data self, requirements: &'data [Requirement]) -> Result<Resolution> {
         let markers = self.interpreter.markers();
+        let python_requirement =
+            PythonRequirement::from_marker_environment(self.interpreter, markers);
         let tags = self.interpreter.tags()?;
         let resolver = Resolver::new(
             Manifest::simple(requirements.to_vec()),
             self.options,
-            markers,
-            self.interpreter,
+            &python_requirement,
+            Some(markers),
             tags,
-            self.client,
             self.flat_index,
             self.index,
             &HashStrategy::None,
             self,
             &EmptyInstalledPackages,
+            DistributionDatabase::new(self.client, self, self.concurrency.downloads),
         )?;
         let graph = resolver.resolve().await.with_context(|| {
             format!(
@@ -189,7 +195,6 @@ impl<'a> BuildContext for BuildDispatch<'a> {
         let Plan {
             cached,
             remote,
-            installed: _,
             reinstalls,
             extraneous: _,
         } = Planner::with_requirements(&resolution.requirements()).build(
@@ -225,8 +230,13 @@ impl<'a> BuildContext for BuildDispatch<'a> {
             vec![]
         } else {
             // TODO(konstin): Check that there is no endless recursion.
-            let downloader =
-                Downloader::new(self.cache, tags, &HashStrategy::None, self.client, self);
+            let downloader = Downloader::new(
+                self.cache,
+                tags,
+                &HashStrategy::None,
+                DistributionDatabase::new(self.client, self, self.concurrency.downloads),
+            );
+
             debug!(
                 "Downloading and building requirement{} for build: {}",
                 if remote.len() == 1 { "" } else { "s" },
@@ -314,8 +324,9 @@ impl<'a> BuildContext for BuildDispatch<'a> {
             self.build_isolation,
             build_kind,
             self.build_extra_env_vars.clone(),
+            self.concurrency.builds,
         )
-        .boxed()
+        .boxed_local()
         .await?;
         Ok(builder)
     }
