@@ -1,26 +1,31 @@
 use std::collections::hash_map::Entry;
 use std::hash::BuildHasherDefault;
+use std::path::Path;
+use std::str::FromStr;
 
 use anyhow::{bail, Result};
 use rustc_hash::FxHashMap;
 use tracing::{debug, warn};
 
-use distribution_types::Hashed;
+use distribution_filename::WheelFilename;
 use distribution_types::{
-    BuiltDist, CachedDirectUrlDist, CachedDist, Dist, IndexLocations, InstalledDist,
-    InstalledMetadata, InstalledVersion, Name, SourceDist,
+    CachedDirectUrlDist, CachedDist, DirectUrlBuiltDist, DirectUrlSourceDist, DirectorySourceDist,
+    Error, GitSourceDist, Hashed, IndexLocations, InstalledDist, InstalledMetadata,
+    InstalledVersion, Name, PathBuiltDist, PathSourceDist, RemoteSource, Requirement,
+    RequirementSource, Verbatim,
 };
-use pep508_rs::{Requirement, VersionOrUrl};
 use platform_tags::Tags;
-use uv_cache::{ArchiveTarget, ArchiveTimestamp, Cache, CacheBucket, WheelCache};
+use uv_cache::{ArchiveTimestamp, Cache, CacheBucket, WheelCache};
 use uv_configuration::{NoBinary, Reinstall};
 use uv_distribution::{
     BuiltWheelIndex, HttpArchivePointer, LocalArchivePointer, RegistryWheelIndex,
 };
 use uv_fs::Simplified;
+use uv_git::GitUrl;
 use uv_interpreter::PythonEnvironment;
 use uv_types::HashStrategy;
 
+use crate::satisfies::RequirementSatisfaction;
 use crate::{ResolvedEditable, SitePackages};
 
 /// A planner to generate an [`Plan`] based on a set of requirements.
@@ -79,7 +84,6 @@ impl<'a> Planner<'a> {
         let mut cached = vec![];
         let mut remote = vec![];
         let mut reinstalls = vec![];
-        let mut installed = vec![];
         let mut extraneous = vec![];
         let mut seen = FxHashMap::with_capacity_and_hasher(
             self.requirements.len(),
@@ -110,7 +114,7 @@ impl<'a> Planner<'a> {
                     debug!("Treating editable requirement as immutable: {installed}");
 
                     // Remove from the site-packages index, to avoid marking as extraneous.
-                    let Some(editable) = installed.as_editable() else {
+                    let Some(editable) = installed.wheel.as_editable() else {
                         warn!("Editable requirement is not editable: {installed}");
                         continue;
                     };
@@ -123,7 +127,7 @@ impl<'a> Planner<'a> {
                 ResolvedEditable::Built(built) => {
                     debug!("Treating editable requirement as mutable: {built}");
 
-                    // Remove any editable installs.
+                    // Remove any editables.
                     let existing = site_packages.remove_editables(built.editable.raw());
                     reinstalls.extend(existing);
 
@@ -138,12 +142,12 @@ impl<'a> Planner<'a> {
 
         for requirement in self.requirements {
             // Filter out incompatible requirements.
-            if !requirement.evaluate_markers(venv.interpreter().markers(), &[]) {
+            if !requirement.evaluate_markers(Some(venv.interpreter().markers()), &[]) {
                 continue;
             }
 
             // If we see the same requirement twice, then we have a conflict.
-            let specifier = Specifier::NonEditable(requirement.version_or_url.as_ref());
+            let specifier = Specifier::NonEditable(&requirement.source);
             match seen.entry(requirement.name.clone()) {
                 Entry::Occupied(value) => {
                     if value.get() == &specifier {
@@ -182,10 +186,15 @@ impl<'a> Planner<'a> {
                 match installed_dists.as_slice() {
                     [] => {}
                     [distribution] => {
-                        if installed_satisfies_requirement(distribution, requirement)? {
-                            debug!("Requirement already installed: {distribution}");
-                            installed.push(distribution.clone());
-                            continue;
+                        match RequirementSatisfaction::check(distribution, &requirement.source)? {
+                            RequirementSatisfaction::Mismatch => {}
+                            RequirementSatisfaction::Satisfied => {
+                                debug!("Requirement already installed: {distribution}");
+                                continue;
+                            }
+                            RequirementSatisfaction::OutOfDate => {
+                                debug!("Requirement installed, but not fresh: {distribution}");
+                            }
                         }
                         reinstalls.push(distribution.clone());
                     }
@@ -197,23 +206,14 @@ impl<'a> Planner<'a> {
             }
 
             if cache.must_revalidate(&requirement.name) {
-                debug!("Must revalidate requirement: {requirement}");
+                debug!("Must revalidate requirement: {}", requirement.name);
                 remote.push(requirement.clone());
                 continue;
             }
 
             // Identify any cached distributions that satisfy the requirement.
-            match requirement.version_or_url.as_ref() {
-                None => {
-                    if let Some((_version, distribution)) =
-                        registry_index.get(&requirement.name).next()
-                    {
-                        debug!("Requirement already cached: {distribution}");
-                        cached.push(CachedDist::Registry(distribution.clone()));
-                        continue;
-                    }
-                }
-                Some(VersionOrUrl::VersionSpecifier(specifier)) => {
+            match &requirement.source {
+                RequirementSource::Registry { specifier, .. } => {
                     if let Some((_version, distribution)) = registry_index
                         .get(&requirement.name)
                         .find(|(version, _)| specifier.contains(version))
@@ -223,40 +223,194 @@ impl<'a> Planner<'a> {
                         continue;
                     }
                 }
-                Some(VersionOrUrl::Url(url)) => {
-                    match Dist::from_url(requirement.name.clone(), url.clone())? {
-                        Dist::Built(BuiltDist::Registry(_)) => {
-                            // Nothing to do.
+                RequirementSource::Url {
+                    subdirectory,
+                    location,
+                    url,
+                } => {
+                    // Check if we have a wheel or a source distribution.
+                    if Path::new(url.path())
+                        .extension()
+                        .is_some_and(|ext| ext.eq_ignore_ascii_case("whl"))
+                    {
+                        // Validate that the name in the wheel matches that of the requirement.
+                        let filename = WheelFilename::from_str(&url.filename()?)?;
+                        if filename.name != requirement.name {
+                            return Err(Error::PackageNameMismatch(
+                                requirement.name.clone(),
+                                filename.name,
+                                url.verbatim().to_string(),
+                            )
+                            .into());
                         }
-                        Dist::Source(SourceDist::Registry(_)) => {
-                            // Nothing to do.
+
+                        let wheel = DirectUrlBuiltDist {
+                            filename,
+                            location: location.clone(),
+                            subdirectory: subdirectory.clone(),
+                            url: url.clone(),
+                        };
+
+                        if !wheel.filename.is_compatible(tags) {
+                            bail!(
+                                "A URL dependency is incompatible with the current platform: {}",
+                                wheel.url
+                            );
                         }
-                        Dist::Built(BuiltDist::DirectUrl(wheel)) => {
-                            if !wheel.filename.is_compatible(tags) {
-                                bail!(
-                                    "A URL dependency is incompatible with the current platform: {}",
-                                    wheel.url
+
+                        if no_binary {
+                            bail!(
+                                "A URL dependency points to a wheel which conflicts with `--no-binary`: {}",
+                                wheel.url
+                            );
+                        }
+
+                        // Find the exact wheel from the cache, since we know the filename in
+                        // advance.
+                        let cache_entry = cache
+                            .shard(
+                                CacheBucket::Wheels,
+                                WheelCache::Url(&wheel.url).wheel_dir(wheel.name().as_ref()),
+                            )
+                            .entry(format!("{}.http", wheel.filename.stem()));
+
+                        // Read the HTTP pointer.
+                        if let Some(pointer) = HttpArchivePointer::read_from(&cache_entry)? {
+                            let archive = pointer.into_archive();
+                            if archive.satisfies(hasher.get(&wheel)) {
+                                let cached_dist = CachedDirectUrlDist::from_url(
+                                    wheel.filename,
+                                    wheel.url,
+                                    archive.hashes,
+                                    cache.archive(&archive.id),
                                 );
+
+                                debug!("URL wheel requirement already cached: {cached_dist}");
+                                cached.push(CachedDist::Url(cached_dist));
+                                continue;
                             }
+                        }
+                    } else {
+                        let sdist = DirectUrlSourceDist {
+                            name: requirement.name.clone(),
+                            location: location.clone(),
+                            subdirectory: subdirectory.clone(),
+                            url: url.clone(),
+                        };
+                        // Find the most-compatible wheel from the cache, since we don't know
+                        // the filename in advance.
+                        if let Some(wheel) = built_index.url(&sdist)? {
+                            let cached_dist = wheel.into_url_dist(url.clone());
+                            debug!("URL source requirement already cached: {cached_dist}");
+                            cached.push(CachedDist::Url(cached_dist));
+                            continue;
+                        }
+                    }
+                }
+                RequirementSource::Git {
+                    repository,
+                    reference,
+                    precise,
+                    subdirectory,
+                    url,
+                } => {
+                    let mut git = GitUrl::new(repository.clone(), reference.clone());
+                    if let Some(precise) = precise {
+                        git = git.with_precise(*precise);
+                    }
+                    let sdist = GitSourceDist {
+                        name: requirement.name.clone(),
+                        git: Box::new(git),
+                        subdirectory: subdirectory.clone(),
+                        url: url.clone(),
+                    };
+                    // Find the most-compatible wheel from the cache, since we don't know
+                    // the filename in advance.
+                    if let Some(wheel) = built_index.git(&sdist) {
+                        let cached_dist = wheel.into_url_dist(url.clone());
+                        debug!("Git source requirement already cached: {cached_dist}");
+                        cached.push(CachedDist::Url(cached_dist));
+                        continue;
+                    }
+                }
+                RequirementSource::Path { url, .. } => {
+                    // Store the canonicalized path, which also serves to validate that it exists.
+                    let path = match url
+                        .to_file_path()
+                        .map_err(|()| Error::UrlFilename(url.to_url()))?
+                        .canonicalize()
+                    {
+                        Ok(path) => path,
+                        Err(err) if err.kind() == std::io::ErrorKind::NotFound => {
+                            return Err(Error::NotFound(url.to_url()).into());
+                        }
+                        Err(err) => return Err(err.into()),
+                    };
 
-                            if no_binary {
-                                bail!(
-                                    "A URL dependency points to a wheel which conflicts with `--no-binary`: {}",
-                                    wheel.url
-                                );
-                            }
+                    // Check if we have a wheel or a source distribution.
+                    if path.is_dir() {
+                        let sdist = DirectorySourceDist {
+                            name: requirement.name.clone(),
+                            url: url.clone(),
+                            path,
+                            editable: false,
+                        };
 
-                            // Find the exact wheel from the cache, since we know the filename in
-                            // advance.
-                            let cache_entry = cache
-                                .shard(
-                                    CacheBucket::Wheels,
-                                    WheelCache::Url(&wheel.url).wheel_dir(wheel.name().as_ref()),
-                                )
-                                .entry(format!("{}.http", wheel.filename.stem()));
+                        // Find the most-compatible wheel from the cache, since we don't know
+                        // the filename in advance.
+                        if let Some(wheel) = built_index.directory(&sdist)? {
+                            let cached_dist = wheel.into_url_dist(url.clone());
+                            debug!("Path source requirement already cached: {cached_dist}");
+                            cached.push(CachedDist::Url(cached_dist));
+                            continue;
+                        }
+                    } else if path
+                        .extension()
+                        .is_some_and(|ext| ext.eq_ignore_ascii_case("whl"))
+                    {
+                        // Validate that the name in the wheel matches that of the requirement.
+                        let filename = WheelFilename::from_str(&url.filename()?)?;
+                        if filename.name != requirement.name {
+                            return Err(Error::PackageNameMismatch(
+                                requirement.name.clone(),
+                                filename.name,
+                                url.verbatim().to_string(),
+                            )
+                            .into());
+                        }
 
-                            // Read the HTTP pointer.
-                            if let Some(pointer) = HttpArchivePointer::read_from(&cache_entry)? {
+                        let wheel = PathBuiltDist {
+                            filename,
+                            url: url.clone(),
+                            path,
+                        };
+
+                        if !wheel.filename.is_compatible(tags) {
+                            bail!(
+                                "A path dependency is incompatible with the current platform: {}",
+                                wheel.path.user_display()
+                            );
+                        }
+
+                        if no_binary {
+                            bail!(
+                                "A path dependency points to a wheel which conflicts with `--no-binary`: {}",
+                                wheel.url
+                            );
+                        }
+
+                        // Find the exact wheel from the cache, since we know the filename in
+                        // advance.
+                        let cache_entry = cache
+                            .shard(
+                                CacheBucket::Wheels,
+                                WheelCache::Url(&wheel.url).wheel_dir(wheel.name().as_ref()),
+                            )
+                            .entry(format!("{}.rev", wheel.filename.stem()));
+
+                        if let Some(pointer) = LocalArchivePointer::read_from(&cache_entry)? {
+                            let timestamp = ArchiveTimestamp::from_file(&wheel.path)?;
+                            if pointer.is_up_to_date(timestamp) {
                                 let archive = pointer.into_archive();
                                 if archive.satisfies(hasher.get(&wheel)) {
                                     let cached_dist = CachedDirectUrlDist::from_url(
@@ -266,86 +420,26 @@ impl<'a> Planner<'a> {
                                         cache.archive(&archive.id),
                                     );
 
-                                    debug!("URL wheel requirement already cached: {cached_dist}");
+                                    debug!("Path wheel requirement already cached: {cached_dist}");
                                     cached.push(CachedDist::Url(cached_dist));
                                     continue;
                                 }
                             }
                         }
-                        Dist::Built(BuiltDist::Path(wheel)) => {
-                            if !wheel.filename.is_compatible(tags) {
-                                bail!(
-                                    "A path dependency is incompatible with the current platform: {}",
-                                    wheel.path.user_display()
-                                );
-                            }
+                    } else {
+                        let sdist = PathSourceDist {
+                            name: requirement.name.clone(),
+                            url: url.clone(),
+                            path,
+                        };
 
-                            if no_binary {
-                                bail!(
-                                    "A path dependency points to a wheel which conflicts with `--no-binary`: {}",
-                                    wheel.url
-                                );
-                            }
-
-                            // Find the exact wheel from the cache, since we know the filename in
-                            // advance.
-                            let cache_entry = cache
-                                .shard(
-                                    CacheBucket::Wheels,
-                                    WheelCache::Url(&wheel.url).wheel_dir(wheel.name().as_ref()),
-                                )
-                                .entry(format!("{}.rev", wheel.filename.stem()));
-
-                            if let Some(pointer) = LocalArchivePointer::read_from(&cache_entry)? {
-                                let timestamp = ArchiveTimestamp::from_file(&wheel.path)?;
-                                if pointer.is_up_to_date(timestamp) {
-                                    let archive = pointer.into_archive();
-                                    if archive.satisfies(hasher.get(&wheel)) {
-                                        let cached_dist = CachedDirectUrlDist::from_url(
-                                            wheel.filename,
-                                            wheel.url,
-                                            archive.hashes,
-                                            cache.archive(&archive.id),
-                                        );
-
-                                        debug!(
-                                            "Path wheel requirement already cached: {cached_dist}"
-                                        );
-                                        cached.push(CachedDist::Url(cached_dist));
-                                        continue;
-                                    }
-                                }
-                            }
-                        }
-                        Dist::Source(SourceDist::DirectUrl(sdist)) => {
-                            // Find the most-compatible wheel from the cache, since we don't know
-                            // the filename in advance.
-                            if let Some(wheel) = built_index.url(&sdist)? {
-                                let cached_dist = wheel.into_url_dist(url.clone());
-                                debug!("URL source requirement already cached: {cached_dist}");
-                                cached.push(CachedDist::Url(cached_dist));
-                                continue;
-                            }
-                        }
-                        Dist::Source(SourceDist::Path(sdist)) => {
-                            // Find the most-compatible wheel from the cache, since we don't know
-                            // the filename in advance.
-                            if let Some(wheel) = built_index.path(&sdist)? {
-                                let cached_dist = wheel.into_url_dist(url.clone());
-                                debug!("Path source requirement already cached: {cached_dist}");
-                                cached.push(CachedDist::Url(cached_dist));
-                                continue;
-                            }
-                        }
-                        Dist::Source(SourceDist::Git(sdist)) => {
-                            // Find the most-compatible wheel from the cache, since we don't know
-                            // the filename in advance.
-                            if let Some(wheel) = built_index.git(&sdist) {
-                                let cached_dist = wheel.into_url_dist(url.clone());
-                                debug!("Git source requirement already cached: {cached_dist}");
-                                cached.push(CachedDist::Url(cached_dist));
-                                continue;
-                            }
+                        // Find the most-compatible wheel from the cache, since we don't know
+                        // the filename in advance.
+                        if let Some(wheel) = built_index.path(&sdist)? {
+                            let cached_dist = wheel.into_url_dist(url.clone());
+                            debug!("Path source requirement already cached: {cached_dist}");
+                            cached.push(CachedDist::Url(cached_dist));
+                            continue;
                         }
                     }
                 }
@@ -378,7 +472,6 @@ impl<'a> Planner<'a> {
 
         Ok(Plan {
             cached,
-            installed,
             remote,
             reinstalls,
             extraneous,
@@ -391,7 +484,7 @@ enum Specifier<'a> {
     /// An editable requirement, marked by the installed version of the package.
     Editable(InstalledVersion<'a>),
     /// A non-editable requirement, marked by the version or URL specifier.
-    NonEditable(Option<&'a VersionOrUrl>),
+    NonEditable(&'a RequirementSource),
 }
 
 #[derive(Debug, Default)]
@@ -400,10 +493,6 @@ pub struct Plan {
     /// available in the local cache.
     pub cached: Vec<CachedDist>,
 
-    /// Any distributions that are already installed in the current environment, and can be used
-    /// to satisfy the requirements.
-    pub installed: Vec<InstalledDist>,
-
     /// The distributions that are not already installed in the current environment, and are
     /// not available in the local cache.
     pub remote: Vec<Requirement>,
@@ -416,53 +505,3 @@ pub struct Plan {
     /// _not_ necessary to satisfy the requirements.
     pub extraneous: Vec<InstalledDist>,
 }
-
-/// Returns true if a requirement is satisfied by an installed distribution.
-///
-/// Returns an error if IO fails during a freshness check for a local path.
-fn installed_satisfies_requirement(
-    distribution: &InstalledDist,
-    requirement: &Requirement,
-) -> Result<bool> {
-    // Filter out already-installed packages.
-    match requirement.version_or_url.as_ref() {
-        // Accept any version of the package.
-        None => return Ok(true),
-
-        // If the requirement comes from a registry, check by name.
-        Some(VersionOrUrl::VersionSpecifier(version_specifier)) => {
-            if version_specifier.contains(distribution.version()) {
-                debug!("Requirement already satisfied: {distribution}");
-                return Ok(true);
-            }
-        }
-
-        // If the requirement comes from a direct URL, check by URL.
-        Some(VersionOrUrl::Url(url)) => {
-            if let InstalledDist::Url(installed) = &distribution {
-                if !installed.editable && &installed.url == url.raw() {
-                    // If the requirement came from a local path, check freshness.
-                    if let Some(archive) = (url.scheme() == "file")
-                        .then(|| url.to_file_path().ok())
-                        .flatten()
-                    {
-                        if ArchiveTimestamp::up_to_date_with(
-                            &archive,
-                            ArchiveTarget::Install(distribution),
-                        )? {
-                            debug!("Requirement already satisfied (and up-to-date): {installed}");
-                            return Ok(true);
-                        }
-                        debug!("Requirement already satisfied (but not up-to-date): {installed}");
-                    } else {
-                        // Otherwise, assume the requirement is up-to-date.
-                        debug!("Requirement already satisfied (assumed up-to-date): {installed}");
-                        return Ok(true);
-                    }
-                }
-            }
-        }
-    }
-
-    Ok(false)
-}
