@@ -7,16 +7,18 @@ use fs_err as fs;
 use rustc_hash::{FxHashMap, FxHashSet};
 use url::Url;
 
-use distribution_types::{InstalledDist, InstalledMetadata, InstalledVersion, Name};
+use distribution_types::{
+    InstalledDist, Name, Requirement, UnresolvedRequirement, UnresolvedRequirementSpecification,
+};
 use pep440_rs::{Version, VersionSpecifiers};
-use pep508_rs::{Requirement, RequirementsTxtRequirement, VerbatimUrl};
-use requirements_txt::{EditableRequirement, RequirementEntry};
+use requirements_txt::EditableRequirement;
 use uv_cache::{ArchiveTarget, ArchiveTimestamp};
 use uv_interpreter::PythonEnvironment;
 use uv_normalize::PackageName;
 use uv_types::InstalledPackagesProvider;
 
 use crate::is_dynamic;
+use crate::satisfies::RequirementSatisfaction;
 
 /// An index over the packages installed in an environment.
 ///
@@ -48,16 +50,21 @@ impl<'a> SitePackages<'a> {
             let site_packages = match fs::read_dir(site_packages) {
                 Ok(site_packages) => {
                     // Collect sorted directory paths; `read_dir` is not stable across platforms
-                    let directories: BTreeSet<_> = site_packages
+                    let dist_likes: BTreeSet<_> = site_packages
                         .filter_map(|read_dir| match read_dir {
                             Ok(entry) => match entry.file_type() {
-                                Ok(file_type) => file_type.is_dir().then_some(Ok(entry.path())),
+                                Ok(file_type) => (file_type.is_dir()
+                                    || entry
+                                        .path()
+                                        .extension()
+                                        .map_or(false, |ext| ext == "egg-link"))
+                                .then_some(Ok(entry.path())),
                                 Err(err) => Some(Err(err)),
                             },
                             Err(err) => Some(Err(err)),
                         })
                         .collect::<Result<_, std::io::Error>>()?;
-                    directories
+                    dist_likes
                 }
                 Err(err) if err.kind() == std::io::ErrorKind::NotFound => {
                     return Ok(Self {
@@ -112,25 +119,6 @@ impl<'a> SitePackages<'a> {
         self.distributions.iter().flatten()
     }
 
-    /// Returns an iterator over the installed distributions, represented as requirements.
-    pub fn requirements(&self) -> impl Iterator<Item = Requirement> + '_ {
-        self.iter().map(|dist| Requirement {
-            name: dist.name().clone(),
-            extras: vec![],
-            version_or_url: Some(match dist.installed_version() {
-                InstalledVersion::Version(version) => {
-                    pep508_rs::VersionOrUrl::VersionSpecifier(pep440_rs::VersionSpecifiers::from(
-                        pep440_rs::VersionSpecifier::equals_version(version.clone()),
-                    ))
-                }
-                InstalledVersion::Url(url, ..) => {
-                    pep508_rs::VersionOrUrl::Url(VerbatimUrl::unknown(url.clone()))
-                }
-            }),
-            marker: None,
-        })
-    }
-
     /// Returns the installed distributions for a given package.
     pub fn get_packages(&self, name: &PackageName) -> Vec<&InstalledDist> {
         let Some(indexes) = self.by_name.get(name) else {
@@ -293,14 +281,14 @@ impl<'a> SitePackages<'a> {
         Ok(diagnostics)
     }
 
-    /// Returns `true` if the installed packages satisfy the given requirements.
+    /// Returns if the installed packages satisfy the given requirements.
     pub fn satisfies(
         &self,
-        requirements: &[RequirementEntry],
+        requirements: &[UnresolvedRequirementSpecification],
         editables: &[EditableRequirement],
         constraints: &[Requirement],
-    ) -> Result<bool> {
-        let mut stack = Vec::<RequirementEntry>::with_capacity(requirements.len());
+    ) -> Result<SatisfiesResult> {
+        let mut stack = Vec::with_capacity(requirements.len());
         let mut seen =
             FxHashSet::with_capacity_and_hasher(requirements.len(), BuildHasherDefault::default());
 
@@ -308,7 +296,7 @@ impl<'a> SitePackages<'a> {
         for entry in requirements {
             if entry
                 .requirement
-                .evaluate_markers(self.venv.interpreter().markers(), &[])
+                .evaluate_markers(Some(self.venv.interpreter().markers()), &[])
             {
                 if seen.insert(entry.clone()) {
                     stack.push(entry.clone());
@@ -322,7 +310,7 @@ impl<'a> SitePackages<'a> {
             match installed.as_slice() {
                 [] => {
                     // The package isn't installed.
-                    return Ok(false);
+                    return Ok(SatisfiesResult::Unsatisfied(requirement.to_string()));
                 }
                 [distribution] => {
                     // Is the editable out-of-date?
@@ -330,12 +318,12 @@ impl<'a> SitePackages<'a> {
                         &requirement.path,
                         ArchiveTarget::Install(distribution),
                     )? {
-                        return Ok(false);
+                        return Ok(SatisfiesResult::Unsatisfied(requirement.to_string()));
                     }
 
                     // Does the editable have dynamic metadata?
                     if is_dynamic(requirement) {
-                        return Ok(false);
+                        return Ok(SatisfiesResult::Unsatisfied(requirement.to_string()));
                     }
 
                     // Recurse into the dependencies.
@@ -349,8 +337,10 @@ impl<'a> SitePackages<'a> {
                             self.venv.interpreter().markers(),
                             &requirement.extras,
                         ) {
-                            let dependency = RequirementEntry {
-                                requirement: RequirementsTxtRequirement::Pep508(dependency),
+                            let dependency = UnresolvedRequirementSpecification {
+                                requirement: UnresolvedRequirement::Named(
+                                    Requirement::from_pep508(dependency)?,
+                                ),
                                 hashes: vec![],
                             };
                             if seen.insert(dependency.clone()) {
@@ -361,7 +351,7 @@ impl<'a> SitePackages<'a> {
                 }
                 _ => {
                     // There are multiple installed distributions for the same package.
-                    return Ok(false);
+                    return Ok(SatisfiesResult::Unsatisfied(requirement.to_string()));
                 }
             }
         }
@@ -369,94 +359,34 @@ impl<'a> SitePackages<'a> {
         // Verify that all non-editable requirements are met.
         while let Some(entry) = stack.pop() {
             let installed = match &entry.requirement {
-                RequirementsTxtRequirement::Pep508(requirement) => {
-                    self.get_packages(&requirement.name)
-                }
-                RequirementsTxtRequirement::Unnamed(requirement) => {
-                    self.get_urls(requirement.url.raw())
-                }
+                UnresolvedRequirement::Named(requirement) => self.get_packages(&requirement.name),
+                UnresolvedRequirement::Unnamed(requirement) => self.get_urls(requirement.url.raw()),
             };
             match installed.as_slice() {
                 [] => {
                     // The package isn't installed.
-                    return Ok(false);
+                    return Ok(SatisfiesResult::Unsatisfied(entry.requirement.to_string()));
                 }
                 [distribution] => {
-                    // Validate that the installed version matches the requirement.
-                    match entry.requirement.version_or_url() {
-                        // Accept any installed version.
-                        None => {}
-
-                        // If the requirement comes from a URL, verify by URL.
-                        Some(pep508_rs::VersionOrUrlRef::Url(url)) => {
-                            let InstalledDist::Url(installed) = &distribution else {
-                                return Ok(false);
-                            };
-
-                            if &installed.url != url.raw() {
-                                return Ok(false);
-                            }
-
-                            // If the requirement came from a local path, check freshness.
-                            if let Ok(archive) = url.to_file_path() {
-                                if !ArchiveTimestamp::up_to_date_with(
-                                    &archive,
-                                    ArchiveTarget::Install(distribution),
-                                )? {
-                                    return Ok(false);
-                                }
-                            }
-                        }
-
-                        Some(pep508_rs::VersionOrUrlRef::VersionSpecifier(version_specifier)) => {
-                            // The installed version doesn't satisfy the requirement.
-                            if !version_specifier.contains(distribution.version()) {
-                                return Ok(false);
-                            }
+                    match RequirementSatisfaction::check(
+                        distribution,
+                        entry.requirement.source()?.as_ref(),
+                    )? {
+                        RequirementSatisfaction::Mismatch | RequirementSatisfaction::OutOfDate => {
+                            return Ok(SatisfiesResult::Unsatisfied(entry.requirement.to_string()))
                         }
+                        RequirementSatisfaction::Satisfied => {}
                     }
-
                     // Validate that the installed version satisfies the constraints.
                     for constraint in constraints {
-                        if constraint.name != *distribution.name() {
-                            continue;
-                        }
-
-                        if !constraint.evaluate_markers(self.venv.interpreter().markers(), &[]) {
-                            continue;
-                        }
-
-                        match &constraint.version_or_url {
-                            // Accept any installed version.
-                            None => {}
-
-                            // If the requirement comes from a URL, verify by URL.
-                            Some(pep508_rs::VersionOrUrl::Url(url)) => {
-                                let InstalledDist::Url(installed) = &distribution else {
-                                    return Ok(false);
-                                };
-
-                                if &installed.url != url.raw() {
-                                    return Ok(false);
-                                }
-
-                                // If the requirement came from a local path, check freshness.
-                                if let Ok(archive) = url.to_file_path() {
-                                    if !ArchiveTimestamp::up_to_date_with(
-                                        &archive,
-                                        ArchiveTarget::Install(distribution),
-                                    )? {
-                                        return Ok(false);
-                                    }
-                                }
-                            }
-
-                            Some(pep508_rs::VersionOrUrl::VersionSpecifier(version_specifier)) => {
-                                // The installed version doesn't satisfy the requirement.
-                                if !version_specifier.contains(distribution.version()) {
-                                    return Ok(false);
-                                }
+                        match RequirementSatisfaction::check(distribution, &constraint.source)? {
+                            RequirementSatisfaction::Mismatch
+                            | RequirementSatisfaction::OutOfDate => {
+                                return Ok(SatisfiesResult::Unsatisfied(
+                                    entry.requirement.to_string(),
+                                ))
                             }
+                            RequirementSatisfaction::Satisfied => {}
                         }
                     }
 
@@ -471,8 +401,10 @@ impl<'a> SitePackages<'a> {
                             self.venv.interpreter().markers(),
                             entry.requirement.extras(),
                         ) {
-                            let dependency = RequirementEntry {
-                                requirement: RequirementsTxtRequirement::Pep508(dependency),
+                            let dependency = UnresolvedRequirementSpecification {
+                                requirement: UnresolvedRequirement::Named(
+                                    Requirement::from_pep508(dependency)?,
+                                ),
                                 hashes: vec![],
                             };
                             if seen.insert(dependency.clone()) {
@@ -483,15 +415,30 @@ impl<'a> SitePackages<'a> {
                 }
                 _ => {
                     // There are multiple installed distributions for the same package.
-                    return Ok(false);
+                    return Ok(SatisfiesResult::Unsatisfied(entry.requirement.to_string()));
                 }
             }
         }
 
-        Ok(true)
+        Ok(SatisfiesResult::Fresh {
+            recursive_requirements: seen,
+        })
     }
 }
 
+/// We check if all requirements are already satisfied, recursing through the requirements tree.
+#[derive(Debug)]
+pub enum SatisfiesResult {
+    /// All requirements are recursively satisfied.
+    Fresh {
+        /// The flattened set (transitive closure) of all requirements checked.
+        recursive_requirements: FxHashSet<UnresolvedRequirementSpecification>,
+    },
+    /// We found an unsatisfied requirement. Since we exit early, we only know about the first
+    /// unsatisfied requirement.
+    Unsatisfied(String),
+}
+
 impl IntoIterator for SitePackages<'_> {
     type Item = InstalledDist;
     type IntoIter = Flatten<std::vec::IntoIter<Option<InstalledDist>>>;
@@ -521,7 +468,7 @@ pub enum Diagnostic {
         /// The package that is missing a dependency.
         package: PackageName,
         /// The dependency that is missing.
-        requirement: Requirement,
+        requirement: pep508_rs::Requirement,
     },
     IncompatibleDependency {
         /// The package that has an incompatible dependency.
@@ -529,7 +476,7 @@ pub enum Diagnostic {
         /// The version of the package that is installed.
         version: Version,
         /// The dependency that is incompatible.
-        requirement: Requirement,
+        requirement: pep508_rs::Requirement,
     },
     DuplicatePackage {
         /// The package that has multiple installed distributions.
@@ -566,14 +513,14 @@ impl Diagnostic {
             } => format!(
                 "The package `{package}` requires `{requirement}`, but `{version}` is installed."
             ),
-            Self::DuplicatePackage { package, paths} => {
+            Self::DuplicatePackage { package, paths } => {
                 let mut paths = paths.clone();
                 paths.sort();
                 format!(
                     "The package `{package}` has multiple installed distributions:{}",
                     paths.iter().fold(String::new(), |acc, path| acc + &format!("\n  - {}", path.display()))
                 )
-            },
+            }
         }
     }
 
