@@ -6,10 +6,12 @@ use std::path::PathBuf;
 use tracing::{debug, instrument};
 
 use uv_cache::Cache;
-use uv_toolchain::PythonVersion;
+use uv_warnings::warn_user_once;
 
 use crate::interpreter::InterpreterInfoError;
-use crate::python_environment::{detect_python_executable, detect_virtual_env};
+use crate::py_launcher::{py_list_paths, Error as PyLauncherError, PyListPath};
+use crate::virtualenv::{detect_virtualenv, virtualenv_python_executable};
+use crate::PythonVersion;
 use crate::{Error, Interpreter};
 
 /// Find a Python of a specific version, a binary with a name or a path to a binary.
@@ -42,7 +44,11 @@ pub fn find_requested_python(request: &str, cache: &Cache) -> Result<Option<Inte
             // SAFETY: Guaranteed by the Ok(versions) guard
             _ => unreachable!(),
         };
-        find_python(selector, cache)
+        let interpreter = find_python(selector, cache)?;
+        interpreter
+            .as_ref()
+            .inspect(|inner| warn_on_unsupported_python(inner));
+        Ok(interpreter)
     } else {
         match fs_err::metadata(request) {
             Ok(metadata) => {
@@ -61,14 +67,18 @@ pub fn find_requested_python(request: &str, cache: &Cache) -> Result<Option<Inte
                     // `-p /home/ferris/.local/bin/python3.10`
                     path
                 };
-                Interpreter::query(executable, cache).map(Some)
+                Interpreter::query(executable, cache)
+                    .inspect(warn_on_unsupported_python)
+                    .map(Some)
             }
             Err(err) if err.kind() == std::io::ErrorKind::NotFound => {
                 // `-p python3.10`; Generally not used on windows because all Python are `python.exe`.
                 let Some(executable) = find_executable(request)? else {
                     return Ok(None);
                 };
-                Interpreter::query(executable, cache).map(Some)
+                Interpreter::query(executable, cache)
+                    .inspect(warn_on_unsupported_python)
+                    .map(Some)
             }
             Err(err) => return Err(err.into()),
         }
@@ -82,13 +92,15 @@ pub fn find_requested_python(request: &str, cache: &Cache) -> Result<Option<Inte
 #[instrument(skip_all)]
 pub fn find_default_python(cache: &Cache) -> Result<Interpreter, Error> {
     debug!("Starting interpreter discovery for default Python");
-    try_find_default_python(cache)?.ok_or(if cfg!(windows) {
-        Error::NoPythonInstalledWindows
-    } else if cfg!(unix) {
-        Error::NoPythonInstalledUnix
-    } else {
-        unreachable!("Only Unix and Windows are supported")
-    })
+    try_find_default_python(cache)?
+        .ok_or(if cfg!(windows) {
+            Error::NoPythonInstalledWindows
+        } else if cfg!(unix) {
+            Error::NoPythonInstalledUnix
+        } else {
+            unreachable!("Only Unix and Windows are supported")
+        })
+        .inspect(warn_on_unsupported_python)
 }
 
 /// Same as [`find_default_python`] but returns `None` if no python is found instead of returning an `Err`.
@@ -138,19 +150,77 @@ fn find_python(
 
                     let interpreter = match Interpreter::query(&path, cache) {
                         Ok(interpreter) => interpreter,
-                        Err(
-                            err @ Error::QueryScript {
-                                err: InterpreterInfoError::UnsupportedPythonVersion,
-                                ..
-                            },
-                        ) => {
-                            if selector.major() <= Some(2) {
-                                return Err(err);
+
+                        // If the Python version is < 3.4, the `-I` flag is not supported, so
+                        // we can't run the script at all, and need to sniff it from the output.
+                        Err(Error::PythonSubcommandOutput { stderr, .. })
+                            if stderr.contains("Unknown option: -I")
+                                && stderr.contains("usage: python [option]") =>
+                        {
+                            // If the user _requested_ a version prior to 3.4, raise an error, as
+                            // 3.4 is the minimum supported version for invoking the interpreter
+                            // query script at all.
+                            match selector {
+                                PythonVersionSelector::Major(major) if major < 3 => {
+                                    return Err(Error::UnsupportedPython(major.to_string()));
+                                }
+                                PythonVersionSelector::MajorMinor(major, minor)
+                                    if (major, minor) < (3, 4) =>
+                                {
+                                    return Err(Error::UnsupportedPython(format!(
+                                        "{major}.{minor}"
+                                    )));
+                                }
+                                PythonVersionSelector::MajorMinorPatch(major, minor, patch)
+                                    if (major, minor) < (3, 4) =>
+                                {
+                                    return Err(Error::UnsupportedPython(format!(
+                                        "{major}.{minor}.{patch}"
+                                    )));
+                                }
+                                _ => {}
                             }
-                            // Skip over Python 2 or older installation when querying for a recent python installation.
-                            debug!("Found a Python 2 installation that isn't supported by uv, skipping.");
+
+                            debug!(
+                                "Found a Python installation that isn't supported by uv, skipping."
+                            );
                             continue;
                         }
+
+                        Err(Error::QueryScript {
+                            err: InterpreterInfoError::UnsupportedPythonVersion { .. },
+                            ..
+                        }) => {
+                            // If the user _requested_ a version prior to 3.7, raise an error, as
+                            // 3.7 is the minimum supported version for running the interpreter
+                            // query script.
+                            match selector {
+                                PythonVersionSelector::Major(major) if major < 3 => {
+                                    return Err(Error::UnsupportedPython(major.to_string()));
+                                }
+                                PythonVersionSelector::MajorMinor(major, minor)
+                                    if (major, minor) < (3, 7) =>
+                                {
+                                    return Err(Error::UnsupportedPython(format!(
+                                        "{major}.{minor}"
+                                    )));
+                                }
+                                PythonVersionSelector::MajorMinorPatch(major, minor, patch)
+                                    if (major, minor) < (3, 7) =>
+                                {
+                                    return Err(Error::UnsupportedPython(format!(
+                                        "{major}.{minor}.{patch}"
+                                    )));
+                                }
+                                _ => {}
+                            }
+
+                            debug!(
+                                "Found a Python installation that isn't supported by uv, skipping."
+                            );
+                            continue;
+                        }
+
                         Err(error) => return Err(error),
                     };
 
@@ -191,7 +261,7 @@ fn find_python(
 
     if cfg!(windows) && !use_override {
         // Use `py` to find the python installation on the system.
-        match windows::py_list_paths() {
+        match py_list_paths() {
             Ok(paths) => {
                 for entry in paths {
                     let installation = PythonInstallation::PyListPath(entry);
@@ -200,19 +270,16 @@ fn find_python(
                     }
                 }
             }
-            Err(Error::PyList(error)) => {
-                if error.kind() == std::io::ErrorKind::NotFound {
-                    debug!("`py` is not installed");
-                }
-            }
-            Err(error) => return Err(error),
+            // Do not error when `py` is not available
+            Err(PyLauncherError::NotFound) => debug!("`py` is not installed"),
+            Err(error) => return Err(Error::PyLauncher(error)),
         }
     }
 
     Ok(None)
 }
 
-/// Find the Python interpreter in `PATH` matching the given name (e.g., `python3`, respecting
+/// Find the Python interpreter in `PATH` matching the given name (e.g., `python3`), respecting
 /// `UV_PYTHON_PATH`.
 ///
 /// Returns `Ok(None)` if not found.
@@ -252,7 +319,7 @@ fn find_executable<R: AsRef<OsStr> + Into<OsString> + Copy>(
 
     if cfg!(windows) && !use_override {
         // Use `py` to find the python installation on the system.
-        match windows::py_list_paths() {
+        match py_list_paths() {
             Ok(paths) => {
                 for entry in paths {
                     // Ex) `--python python3.12.exe`
@@ -270,25 +337,15 @@ fn find_executable<R: AsRef<OsStr> + Into<OsString> + Copy>(
                     }
                 }
             }
-            Err(Error::PyList(error)) => {
-                if error.kind() == std::io::ErrorKind::NotFound {
-                    debug!("`py` is not installed");
-                }
-            }
-            Err(error) => return Err(error),
+            // Do not error when `py` is not available
+            Err(PyLauncherError::NotFound) => debug!("`py` is not installed"),
+            Err(error) => return Err(Error::PyLauncher(error)),
         }
     }
 
     Ok(None)
 }
 
-#[derive(Debug, Clone)]
-struct PyListPath {
-    major: u8,
-    minor: u8,
-    executable_path: PathBuf,
-}
-
 #[derive(Debug, Clone)]
 enum PythonInstallation {
     PyListPath(PyListPath),
@@ -401,14 +458,15 @@ impl PythonVersionSelector {
             ],
         }
     }
+}
 
-    fn major(self) -> Option<u8> {
-        match self {
-            Self::Default => None,
-            Self::Major(major) => Some(major),
-            Self::MajorMinor(major, _) => Some(major),
-            Self::MajorMinorPatch(major, _, _) => Some(major),
-        }
+fn warn_on_unsupported_python(interpreter: &Interpreter) {
+    // Warn on usage with an unsupported Python version
+    if interpreter.python_tuple() < (3, 8) {
+        warn_user_once!(
+            "uv is only compatible with Python 3.8+, found Python {}.",
+            interpreter.python_version()
+        );
     }
 }
 
@@ -439,6 +497,7 @@ pub fn find_best_python(
 
     // First, check for an exact match (or the first available version if no Python version was provided)
     if let Some(interpreter) = find_version(python_version, system, cache)? {
+        warn_on_unsupported_python(&interpreter);
         return Ok(interpreter);
     }
 
@@ -449,6 +508,7 @@ pub fn find_best_python(
             if let Some(interpreter) =
                 find_version(Some(&python_version.without_patch()), system, cache)?
             {
+                warn_on_unsupported_python(&interpreter);
                 return Ok(interpreter);
             }
         }
@@ -495,8 +555,8 @@ fn find_version(
 
     // Check if the venv Python matches.
     if !system {
-        if let Some(venv) = detect_virtual_env()? {
-            let executable = detect_python_executable(venv);
+        if let Some(venv) = detect_virtualenv()? {
+            let executable = virtualenv_python_executable(venv);
             let interpreter = Interpreter::query(executable, cache)?;
 
             if version_matches(&interpreter) {
@@ -522,75 +582,6 @@ fn find_version(
 }
 
 mod windows {
-    use std::path::PathBuf;
-    use std::process::Command;
-
-    use once_cell::sync::Lazy;
-    use regex::Regex;
-    use tracing::info_span;
-
-    use crate::find_python::PyListPath;
-    use crate::Error;
-
-    /// ```text
-    /// -V:3.12          C:\Users\Ferris\AppData\Local\Programs\Python\Python312\python.exe
-    /// -V:3.8           C:\Users\Ferris\AppData\Local\Programs\Python\Python38\python.exe
-    /// ```
-    static PY_LIST_PATHS: Lazy<Regex> = Lazy::new(|| {
-        // Without the `R` flag, paths have trailing \r
-        Regex::new(r"(?mR)^ -(?:V:)?(\d).(\d+)-?(?:arm)?\d*\s*\*?\s*(.*)$").unwrap()
-    });
-
-    /// Run `py --list-paths` to find the installed pythons.
-    ///
-    /// The command takes 8ms on my machine.
-    /// TODO(konstin): Implement <https://peps.python.org/pep-0514/> to read python installations from the registry instead.
-    pub(super) fn py_list_paths() -> Result<Vec<PyListPath>, Error> {
-        let output = info_span!("py_list_paths")
-            .in_scope(|| Command::new("py").arg("--list-paths").output())
-            .map_err(Error::PyList)?;
-
-        // `py` sometimes prints "Installed Pythons found by py Launcher for Windows" to stderr which we ignore.
-        if !output.status.success() {
-            return Err(Error::PythonSubcommandOutput {
-                message: format!(
-                    "Running `py --list-paths` failed with status {}",
-                    output.status
-                ),
-                exit_code: output.status,
-                stdout: String::from_utf8_lossy(&output.stdout).trim().to_string(),
-                stderr: String::from_utf8_lossy(&output.stderr).trim().to_string(),
-            });
-        }
-
-        // Find the first python of the version we want in the list
-        let stdout =
-            String::from_utf8(output.stdout).map_err(|err| Error::PythonSubcommandOutput {
-                message: format!("The stdout of `py --list-paths` isn't UTF-8 encoded: {err}"),
-                exit_code: output.status,
-                stdout: String::from_utf8_lossy(err.as_bytes()).trim().to_string(),
-                stderr: String::from_utf8_lossy(&output.stderr).trim().to_string(),
-            })?;
-
-        Ok(PY_LIST_PATHS
-            .captures_iter(&stdout)
-            .filter_map(|captures| {
-                let (_, [major, minor, path]) = captures.extract();
-                if let (Some(major), Some(minor)) =
-                    (major.parse::<u8>().ok(), minor.parse::<u8>().ok())
-                {
-                    Some(PyListPath {
-                        major,
-                        minor,
-                        executable_path: PathBuf::from(path),
-                    })
-                } else {
-                    None
-                }
-            })
-            .collect())
-    }
-
     /// On Windows we might encounter the Windows Store proxy shim (enabled in:
     /// Settings/Apps/Advanced app settings/App execution aliases). When Python is _not_ installed
     /// via the Windows Store, but the proxy shim is enabled, then executing `python.exe` or
@@ -745,7 +736,7 @@ mod windows {
                     ));
             assert_snapshot!(
                 format_err(result),
-                @"Failed to locate Python interpreter at `C:\\does\\not\\exists\\python3.12`"
+                @"Failed to locate Python interpreter at: `C:\\does\\not\\exists\\python3.12`"
             );
         }
     }
@@ -802,6 +793,6 @@ mod tests {
                 "/does/not/exists/python3.12".to_string(),
             ));
         assert_snapshot!(
-            format_err(result), @"Failed to locate Python interpreter at `/does/not/exists/python3.12`");
+            format_err(result), @"Failed to locate Python interpreter at: `/does/not/exists/python3.12`");
     }
 }
