@@ -31,6 +31,7 @@ pub struct Interpreter {
     base_prefix: PathBuf,
     base_executable: Option<PathBuf>,
     sys_executable: PathBuf,
+    sys_path: Vec<PathBuf>,
     stdlib: PathBuf,
     tags: OnceCell<Tags>,
     target: Option<Target>,
@@ -59,6 +60,7 @@ impl Interpreter {
             base_prefix: info.base_prefix,
             base_executable: info.base_executable,
             sys_executable: info.sys_executable,
+            sys_path: info.sys_path,
             stdlib: info.stdlib,
             tags: OnceCell::new(),
             target: None,
@@ -89,6 +91,7 @@ impl Interpreter {
             base_prefix: PathBuf::from("/dev/null"),
             base_executable: None,
             sys_executable: PathBuf::from("/dev/null"),
+            sys_path: vec![],
             stdlib: PathBuf::from("/dev/null"),
             tags: OnceCell::new(),
             target: None,
@@ -202,31 +205,31 @@ impl Interpreter {
 
     /// Returns the Python version.
     #[inline]
-    pub const fn python_version(&self) -> &Version {
-        &self.markers.python_full_version.version
+    pub fn python_version(&self) -> &Version {
+        &self.markers.python_full_version().version
     }
 
     /// Returns the `python_full_version` marker corresponding to this Python version.
     #[inline]
-    pub const fn python_full_version(&self) -> &StringVersion {
-        &self.markers.python_full_version
+    pub fn python_full_version(&self) -> &StringVersion {
+        self.markers.python_full_version()
     }
 
     /// Return the major version of this Python version.
     pub fn python_major(&self) -> u8 {
-        let major = self.markers.python_full_version.version.release()[0];
+        let major = self.markers.python_full_version().version.release()[0];
         u8::try_from(major).expect("invalid major version")
     }
 
     /// Return the minor version of this Python version.
     pub fn python_minor(&self) -> u8 {
-        let minor = self.markers.python_full_version.version.release()[1];
+        let minor = self.markers.python_full_version().version.release()[1];
         u8::try_from(minor).expect("invalid minor version")
     }
 
     /// Return the patch version of this Python version.
     pub fn python_patch(&self) -> u8 {
-        let minor = self.markers.python_full_version.version.release()[2];
+        let minor = self.markers.python_full_version().version.release()[2];
         u8::try_from(minor).expect("invalid patch version")
     }
 
@@ -237,13 +240,13 @@ impl Interpreter {
 
     /// Return the major version of the implementation (e.g., `CPython` or `PyPy`).
     pub fn implementation_major(&self) -> u8 {
-        let major = self.markers.implementation_version.version.release()[0];
+        let major = self.markers.implementation_version().version.release()[0];
         u8::try_from(major).expect("invalid major version")
     }
 
     /// Return the minor version of the implementation (e.g., `CPython` or `PyPy`).
     pub fn implementation_minor(&self) -> u8 {
-        let minor = self.markers.implementation_version.version.release()[1];
+        let minor = self.markers.implementation_version().version.release()[1];
         u8::try_from(minor).expect("invalid minor version")
     }
 
@@ -254,7 +257,7 @@ impl Interpreter {
 
     /// Returns the implementation name (e.g., `CPython` or `PyPy`).
     pub fn implementation_name(&self) -> &str {
-        &self.markers.implementation_name
+        self.markers.implementation_name()
     }
 
     /// Return the `sys.base_exec_prefix` path for this Python interpreter.
@@ -283,6 +286,11 @@ impl Interpreter {
         &self.sys_executable
     }
 
+    /// Return the `sys.path` for this Python interpreter.
+    pub fn sys_path(&self) -> &Vec<PathBuf> {
+        &self.sys_path
+    }
+
     /// Return the `stdlib` path for this Python interpreter, as returned by `sysconfig.get_paths()`.
     pub fn stdlib(&self) -> &Path {
         &self.stdlib
@@ -337,7 +345,7 @@ impl Interpreter {
         Layout {
             python_version: self.python_tuple(),
             sys_executable: self.sys_executable().to_path_buf(),
-            os_name: self.markers.os_name.clone(),
+            os_name: self.markers.os_name().to_string(),
             scheme: if let Some(target) = self.target.as_ref() {
                 target.scheme()
             } else {
@@ -419,6 +427,7 @@ struct InterpreterInfo {
     base_prefix: PathBuf,
     base_executable: Option<PathBuf>,
     sys_executable: PathBuf,
+    sys_path: Vec<PathBuf>,
     stdlib: PathBuf,
     gil_disabled: bool,
 }
@@ -540,7 +549,7 @@ impl InterpreterInfo {
                         if cached.timestamp == modified {
                             debug!(
                                 "Cached interpreter info for Python {}, skipping probing: {}",
-                                cached.data.markers.python_full_version,
+                                cached.data.markers.python_full_version(),
                                 executable.user_display()
                             );
                             return Ok(cached.data);
@@ -567,7 +576,7 @@ impl InterpreterInfo {
         let info = Self::query(executable, cache)?;
         debug!(
             "Found Python {} for: {}",
-            info.markers.python_full_version,
+            info.markers.python_full_version(),
             executable.display()
         );
 
@@ -634,6 +643,10 @@ mod tests {
                 "base_prefix": "/home/ferris/.pyenv/versions/3.12.0",
                 "prefix": "/home/ferris/projects/uv/.venv",
                 "sys_executable": "/home/ferris/projects/uv/.venv/bin/python",
+                "sys_path": [
+                    "/home/ferris/.pyenv/versions/3.12.0/lib/python3.12/lib/python3.12",
+                    "/home/ferris/.pyenv/versions/3.12.0/lib/python3.12/site-packages"
+                ],
                 "stdlib": "/home/ferris/.pyenv/versions/3.12.0/lib/python3.12",
                 "scheme": {
                     "data": "/home/ferris/.pyenv/versions/3.12.0",
@@ -670,7 +683,7 @@ mod tests {
         .unwrap();
         let interpreter = Interpreter::query(&mocked_interpreter, &cache).unwrap();
         assert_eq!(
-            interpreter.markers.python_version.version,
+            interpreter.markers.python_version().version,
             Version::from_str("3.12").unwrap()
         );
         fs::write(
@@ -683,7 +696,7 @@ mod tests {
         .unwrap();
         let interpreter = Interpreter::query(&mocked_interpreter, &cache).unwrap();
         assert_eq!(
-            interpreter.markers.python_version.version,
+            interpreter.markers.python_version().version,
             Version::from_str("3.13").unwrap()
         );
     }
