@@ -14,28 +14,38 @@ use std::process::ExitStatus;
 
 use thiserror::Error;
 
-pub use crate::cfg::PyVenvConfiguration;
+use uv_fs::Simplified;
+
+pub use crate::environment::PythonEnvironment;
 pub use crate::find_python::{find_best_python, find_default_python, find_requested_python};
 pub use crate::interpreter::Interpreter;
 use crate::interpreter::InterpreterInfoError;
-pub use crate::python_environment::PythonEnvironment;
-pub use crate::virtualenv::Virtualenv;
+pub use crate::python_version::PythonVersion;
+pub use crate::target::Target;
+pub use crate::virtualenv::{PyVenvConfiguration, VirtualEnvironment};
 
-mod cfg;
+mod environment;
 mod find_python;
+mod implementation;
 mod interpreter;
-mod python_environment;
+pub mod managed;
+pub mod platform;
+mod py_launcher;
+mod python_version;
+mod target;
 mod virtualenv;
 
 #[derive(Debug, Error)]
 pub enum Error {
-    #[error("Expected `{0}` to be a virtualenv, but `pyvenv.cfg` is missing")]
+    #[error("Expected `{}` to be a virtualenv, but `pyvenv.cfg` is missing", _0.user_display())]
     MissingPyVenvCfg(PathBuf),
     #[error("No versions of Python could be found. Is Python installed?")]
     PythonNotFound,
     #[error("Failed to locate a virtualenv or Conda environment (checked: `VIRTUAL_ENV`, `CONDA_PREFIX`, and `.venv`). Run `uv venv` to create a virtualenv.")]
     VenvNotFound,
-    #[error("Failed to locate Python interpreter at `{0}`")]
+    #[error("Virtualenv does not exist at: `{}`", _0.user_display())]
+    VenvDoesNotExist(PathBuf),
+    #[error("Failed to locate Python interpreter at: `{0}`")]
     RequestedPythonNotFound(String),
     #[error(transparent)]
     Io(#[from] io::Error),
@@ -45,8 +55,8 @@ pub enum Error {
         #[source]
         err: io::Error,
     },
-    #[error("Failed to run `py --list-paths` to find Python installations. Is Python installed?")]
-    PyList(#[source] io::Error),
+    #[error(transparent)]
+    PyLauncher(#[from] py_launcher::Error),
     #[cfg(windows)]
     #[error(
         "No Python {0} found through `py --list-paths` or in `PATH`. Is Python {0} installed?"
@@ -68,10 +78,12 @@ pub enum Error {
         stdout: String,
         stderr: String,
     },
+    #[error("Requested Python version ({0}) is unsupported")]
+    UnsupportedPython(String),
     #[error("Failed to write to cache")]
     Encode(#[from] rmp_serde::encode::Error),
     #[error("Broken virtualenv: Failed to parse pyvenv.cfg")]
-    Cfg(#[from] cfg::Error),
+    Cfg(#[from] virtualenv::Error),
     #[error("Error finding `{}` in PATH", _0.to_string_lossy())]
     WhichError(OsString, #[source] which::Error),
     #[error("Can't use Python at `{interpreter}`")]
