@@ -1,9 +1,10 @@
-//! Reading from `pyproject.toml`
-//! * `project.{dependencies,optional-dependencies}`,
-//! * `tool.uv.sources` and
+//! Reads the following fields from from `pyproject.toml`:
+//!
+//! * `project.{dependencies,optional-dependencies}`
+//! * `tool.uv.sources`
 //! * `tool.uv.workspace`
 //!
-//! and lowering them into a dependency specification.
+//! Then lowers them into a dependency specification.
 
 use std::collections::HashMap;
 use std::io;
@@ -21,11 +22,12 @@ use url::Url;
 
 use distribution_types::{ParsedUrlError, Requirement, RequirementSource, Requirements};
 use pep440_rs::VersionSpecifiers;
-use pep508_rs::{VerbatimUrl, VersionOrUrl};
+use pep508_rs::{RequirementOrigin, VerbatimUrl, VersionOrUrl};
 use uv_configuration::PreviewMode;
 use uv_fs::Simplified;
 use uv_git::GitReference;
 use uv_normalize::{ExtraName, PackageName};
+use uv_warnings::warn_user_once;
 
 use crate::ExtrasSpecification;
 
@@ -75,7 +77,7 @@ pub enum LoweringError {
 pub struct PyProjectToml {
     /// PEP 621-compliant project metadata.
     pub project: Option<Project>,
-    /// Proprietary additions.
+    /// Tool-specific metadata.
     pub tool: Option<Tool>,
 }
 
@@ -99,26 +101,21 @@ pub struct Project {
     pub dynamic: Option<Vec<String>>,
 }
 
-/// `tool`.
 #[derive(Serialize, Deserialize, Debug, Clone, PartialEq, Eq)]
 #[cfg_attr(feature = "schemars", derive(schemars::JsonSchema))]
 pub struct Tool {
     pub uv: Option<ToolUv>,
 }
 
-/// `tool.uv`.
 #[derive(Serialize, Deserialize, Debug, Clone, PartialEq, Eq)]
 #[cfg_attr(feature = "schemars", derive(schemars::JsonSchema))]
-#[serde(deny_unknown_fields)]
 pub struct ToolUv {
     pub sources: Option<HashMap<PackageName, Source>>,
     pub workspace: Option<ToolUvWorkspace>,
 }
 
-/// `tool.uv.workspace`.
 #[derive(Serialize, Deserialize, Debug, Clone, PartialEq, Eq)]
 #[cfg_attr(feature = "schemars", derive(schemars::JsonSchema))]
-#[serde(deny_unknown_fields)]
 pub struct ToolUvWorkspace {
     pub members: Option<Vec<SerdePattern>>,
     pub exclude: Option<Vec<SerdePattern>>,
@@ -239,6 +236,7 @@ impl Pep621Metadata {
     pub(crate) fn try_from(
         pyproject: PyProjectToml,
         extras: &ExtrasSpecification,
+        pyproject_path: &Path,
         project_dir: &Path,
         workspace_sources: &HashMap<PackageName, Source>,
         workspace_packages: &HashMap<PackageName, String>,
@@ -282,6 +280,8 @@ impl Pep621Metadata {
         let requirements = lower_requirements(
             &project.dependencies.unwrap_or_default(),
             &project.optional_dependencies.unwrap_or_default(),
+            pyproject_path,
+            &project.name,
             project_dir,
             &project_sources.unwrap_or_default(),
             workspace_sources,
@@ -320,6 +320,8 @@ impl Pep621Metadata {
 pub(crate) fn lower_requirements(
     dependencies: &[String],
     optional_dependencies: &IndexMap<ExtraName, Vec<String>>,
+    pyproject_path: &Path,
+    project_name: &PackageName,
     project_dir: &Path,
     project_sources: &HashMap<PackageName, Source>,
     workspace_sources: &HashMap<PackageName, Source>,
@@ -329,10 +331,13 @@ pub(crate) fn lower_requirements(
     let dependencies = dependencies
         .iter()
         .map(|dependency| {
-            let requirement = pep508_rs::Requirement::from_str(dependency)?;
+            let requirement = pep508_rs::Requirement::from_str(dependency)?.with_origin(
+                RequirementOrigin::Project(pyproject_path.to_path_buf(), project_name.clone()),
+            );
             let name = requirement.name.clone();
             lower_requirement(
                 requirement,
+                project_name,
                 project_dir,
                 project_sources,
                 workspace_sources,
@@ -348,10 +353,16 @@ pub(crate) fn lower_requirements(
             let dependencies: Vec<_> = dependencies
                 .iter()
                 .map(|dependency| {
-                    let requirement = pep508_rs::Requirement::from_str(dependency)?;
+                    let requirement = pep508_rs::Requirement::from_str(dependency)?.with_origin(
+                        RequirementOrigin::Project(
+                            pyproject_path.to_path_buf(),
+                            project_name.clone(),
+                        ),
+                    );
                     let name = requirement.name.clone();
                     lower_requirement(
                         requirement,
+                        project_name,
                         project_dir,
                         project_sources,
                         workspace_sources,
@@ -373,6 +384,7 @@ pub(crate) fn lower_requirements(
 /// Combine `project.dependencies` or `project.optional-dependencies` with `tool.uv.sources`.
 pub(crate) fn lower_requirement(
     requirement: pep508_rs::Requirement,
+    project_name: &PackageName,
     project_dir: &Path,
     project_sources: &HashMap<PackageName, Source>,
     workspace_sources: &HashMap<PackageName, Source>,
@@ -397,7 +409,15 @@ pub(crate) fn lower_requirement(
     }
 
     let Some(source) = source else {
+        let has_sources = !project_sources.is_empty() || !workspace_sources.is_empty();
         // Support recursive editable inclusions.
+        if has_sources && requirement.version_or_url.is_none() && &requirement.name != project_name
+        {
+            warn_user_once!(
+                "Missing version constraint (e.g., a lower bound) for `{}`",
+                requirement.name
+            );
+        }
         return Ok(Requirement::from_pep508(requirement)?);
     };
 
@@ -448,6 +468,7 @@ pub(crate) fn lower_requirement(
                 url,
                 repository,
                 reference,
+                precise: None,
                 subdirectory: subdirectory.map(PathBuf::from),
             }
         }
@@ -478,10 +499,16 @@ pub(crate) fn lower_requirement(
             path_source(path, project_dir, editable)?
         }
         Source::Registry { index } => match requirement.version_or_url {
-            None => RequirementSource::Registry {
-                specifier: VersionSpecifiers::empty(),
-                index: Some(index),
-            },
+            None => {
+                warn_user_once!(
+                    "Missing version constraint (e.g., a lower bound) for `{}`",
+                    requirement.name
+                );
+                RequirementSource::Registry {
+                    specifier: VersionSpecifiers::empty(),
+                    index: Some(index),
+                }
+            }
             Some(VersionOrUrl::VersionSpecifier(version)) => RequirementSource::Registry {
                 specifier: version,
                 index: Some(index),
@@ -514,6 +541,7 @@ pub(crate) fn lower_requirement(
         extras: requirement.extras,
         marker: requirement.marker,
         source,
+        origin: requirement.origin,
     })
 }
 
@@ -632,8 +660,8 @@ mod test {
     use anyhow::Context;
     use indoc::indoc;
     use insta::assert_snapshot;
-    use uv_configuration::PreviewMode;
 
+    use uv_configuration::PreviewMode;
     use uv_fs::Simplified;
 
     use crate::{ExtrasSpecification, RequirementsSpecification};
@@ -758,6 +786,20 @@ mod test {
         "###);
     }
 
+    #[test]
+    fn missing_constraint() {
+        let input = indoc! {r#"
+            [project]
+            name = "foo"
+            version = "0.0.0"
+            dependencies = [
+              "tqdm",
+            ]
+        "#};
+
+        assert!(from_source(input, "pyproject.toml", &ExtrasSpecification::None).is_ok());
+    }
+
     #[test]
     fn invalid_syntax() {
         let input = indoc! {r#"
