@@ -4,30 +4,27 @@ use std::str::FromStr;
 
 use anyhow::Result;
 use configparser::ini::Ini;
-use futures::{StreamExt, TryStreamExt};
+use futures::{stream::FuturesOrdered, TryStreamExt};
 use serde::Deserialize;
 use tracing::debug;
 
 use distribution_filename::{SourceDistFilename, WheelFilename};
 use distribution_types::{
-    BuildableSource, DirectSourceUrl, GitSourceUrl, PathSourceUrl, RemoteSource, SourceUrl,
-    VersionId,
-};
-use pep508_rs::{
-    Requirement, RequirementsTxtRequirement, Scheme, UnnamedRequirement, VersionOrUrl,
+    BuildableSource, DirectSourceUrl, DirectorySourceUrl, GitSourceUrl, PathSourceUrl,
+    RemoteSource, Requirement, SourceUrl, UnresolvedRequirement,
+    UnresolvedRequirementSpecification, VersionId,
 };
+use pep508_rs::{Scheme, UnnamedRequirement, VersionOrUrl};
 use pypi_types::Metadata10;
-use requirements_txt::RequirementEntry;
-use uv_client::RegistryClient;
 use uv_distribution::{DistributionDatabase, Reporter};
 use uv_normalize::PackageName;
 use uv_resolver::{InMemoryIndex, MetadataResponse};
 use uv_types::{BuildContext, HashStrategy};
 
 /// Like [`RequirementsSpecification`], but with concrete names for all requirements.
-pub struct NamedRequirementsResolver<'a, Context: BuildContext + Send + Sync> {
+pub struct NamedRequirementsResolver<'a, Context: BuildContext> {
     /// The requirements for the project.
-    requirements: Vec<RequirementEntry>,
+    requirements: Vec<UnresolvedRequirementSpecification>,
     /// Whether to check hashes for distributions.
     hasher: &'a HashStrategy,
     /// The in-memory index for resolving dependencies.
@@ -36,20 +33,19 @@ pub struct NamedRequirementsResolver<'a, Context: BuildContext + Send + Sync> {
     database: DistributionDatabase<'a, Context>,
 }
 
-impl<'a, Context: BuildContext + Send + Sync> NamedRequirementsResolver<'a, Context> {
+impl<'a, Context: BuildContext> NamedRequirementsResolver<'a, Context> {
     /// Instantiate a new [`NamedRequirementsResolver`] for a given set of requirements.
     pub fn new(
-        requirements: Vec<RequirementEntry>,
+        requirements: Vec<UnresolvedRequirementSpecification>,
         hasher: &'a HashStrategy,
-        context: &'a Context,
-        client: &'a RegistryClient,
         index: &'a InMemoryIndex,
+        database: DistributionDatabase<'a, Context>,
     ) -> Self {
         Self {
             requirements,
             hasher,
             index,
-            database: DistributionDatabase::new(client, context),
+            database,
         }
     }
 
@@ -70,16 +66,17 @@ impl<'a, Context: BuildContext + Send + Sync> NamedRequirementsResolver<'a, Cont
             index,
             database,
         } = self;
-        futures::stream::iter(requirements)
+        requirements
+            .into_iter()
             .map(|entry| async {
                 match entry.requirement {
-                    RequirementsTxtRequirement::Pep508(requirement) => Ok(requirement),
-                    RequirementsTxtRequirement::Unnamed(requirement) => {
-                        Self::resolve_requirement(requirement, hasher, index, &database).await
-                    }
+                    UnresolvedRequirement::Named(requirement) => Ok(requirement),
+                    UnresolvedRequirement::Unnamed(requirement) => Ok(Requirement::from_pep508(
+                        Self::resolve_requirement(requirement, hasher, index, &database).await?,
+                    )?),
                 }
             })
-            .buffered(50)
+            .collect::<FuturesOrdered<_>>()
             .try_collect()
             .await
     }
@@ -90,7 +87,7 @@ impl<'a, Context: BuildContext + Send + Sync> NamedRequirementsResolver<'a, Cont
         hasher: &HashStrategy,
         index: &InMemoryIndex,
         database: &DistributionDatabase<'a, Context>,
-    ) -> Result<Requirement> {
+    ) -> Result<pep508_rs::Requirement> {
         // If the requirement is a wheel, extract the package name from the wheel filename.
         //
         // Ex) `anyio-4.3.0-py3-none-any.whl`
@@ -99,11 +96,12 @@ impl<'a, Context: BuildContext + Send + Sync> NamedRequirementsResolver<'a, Cont
             .is_some_and(|ext| ext.eq_ignore_ascii_case("whl"))
         {
             let filename = WheelFilename::from_str(&requirement.url.filename()?)?;
-            return Ok(Requirement {
+            return Ok(pep508_rs::Requirement {
                 name: filename.name,
                 extras: requirement.extras,
                 version_or_url: Some(VersionOrUrl::Url(requirement.url)),
                 marker: requirement.marker,
+                origin: requirement.origin,
             });
         }
 
@@ -117,11 +115,12 @@ impl<'a, Context: BuildContext + Send + Sync> NamedRequirementsResolver<'a, Cont
             .ok()
             .and_then(|filename| SourceDistFilename::parsed_normalized_filename(&filename).ok())
         {
-            return Ok(Requirement {
+            return Ok(pep508_rs::Requirement {
                 name: filename.name,
                 extras: requirement.extras,
                 version_or_url: Some(VersionOrUrl::Url(requirement.url)),
                 marker: requirement.marker,
+                origin: requirement.origin,
             });
         }
 
@@ -144,16 +143,18 @@ impl<'a, Context: BuildContext + Send + Sync> NamedRequirementsResolver<'a, Cont
                             path = path.display(),
                             name = metadata.name
                         );
-                        return Ok(Requirement {
+                        return Ok(pep508_rs::Requirement {
                             name: metadata.name,
                             extras: requirement.extras,
                             version_or_url: Some(VersionOrUrl::Url(requirement.url)),
                             marker: requirement.marker,
+                            origin: requirement.origin,
                         });
                     }
 
                     // Attempt to read a `pyproject.toml` file.
-                    if let Some(pyproject) = fs_err::read_to_string(path.join("pyproject.toml"))
+                    let project_path = path.join("pyproject.toml");
+                    if let Some(pyproject) = fs_err::read_to_string(project_path)
                         .ok()
                         .and_then(|contents| toml::from_str::<PyProjectToml>(&contents).ok())
                     {
@@ -164,11 +165,12 @@ impl<'a, Context: BuildContext + Send + Sync> NamedRequirementsResolver<'a, Cont
                                 path = path.display(),
                                 name = project.name
                             );
-                            return Ok(Requirement {
+                            return Ok(pep508_rs::Requirement {
                                 name: project.name,
                                 extras: requirement.extras,
                                 version_or_url: Some(VersionOrUrl::Url(requirement.url)),
                                 marker: requirement.marker,
+                                origin: requirement.origin,
                             });
                         }
 
@@ -181,11 +183,12 @@ impl<'a, Context: BuildContext + Send + Sync> NamedRequirementsResolver<'a, Cont
                                         path = path.display(),
                                         name = name
                                     );
-                                    return Ok(Requirement {
+                                    return Ok(pep508_rs::Requirement {
                                         name,
                                         extras: requirement.extras,
                                         version_or_url: Some(VersionOrUrl::Url(requirement.url)),
                                         marker: requirement.marker,
+                                        origin: requirement.origin,
                                     });
                                 }
                             }
@@ -209,29 +212,37 @@ impl<'a, Context: BuildContext + Send + Sync> NamedRequirementsResolver<'a, Cont
                                         path = path.display(),
                                         name = name
                                     );
-                                    return Ok(Requirement {
+                                    return Ok(pep508_rs::Requirement {
                                         name,
                                         extras: requirement.extras,
                                         version_or_url: Some(VersionOrUrl::Url(requirement.url)),
                                         marker: requirement.marker,
+                                        origin: requirement.origin,
                                     });
                                 }
                             }
                         }
                     }
-                }
 
-                SourceUrl::Path(PathSourceUrl {
-                    url: &requirement.url,
-                    path: Cow::Owned(path),
-                })
+                    SourceUrl::Directory(DirectorySourceUrl {
+                        url: &requirement.url,
+                        path: Cow::Owned(path),
+                    })
+                } else {
+                    SourceUrl::Path(PathSourceUrl {
+                        url: &requirement.url,
+                        path: Cow::Owned(path),
+                    })
+                }
             }
             Some(Scheme::Http | Scheme::Https) => SourceUrl::Direct(DirectSourceUrl {
                 url: &requirement.url,
             }),
-            Some(Scheme::GitSsh | Scheme::GitHttps) => SourceUrl::Git(GitSourceUrl {
-                url: &requirement.url,
-            }),
+            Some(Scheme::GitSsh | Scheme::GitHttps | Scheme::GitHttp) => {
+                SourceUrl::Git(GitSourceUrl {
+                    url: &requirement.url,
+                })
+            }
             _ => {
                 return Err(anyhow::anyhow!(
                     "Unsupported scheme for unnamed requirement: {}",
@@ -267,11 +278,12 @@ impl<'a, Context: BuildContext + Send + Sync> NamedRequirementsResolver<'a, Cont
             }
         };
 
-        Ok(Requirement {
+        Ok(pep508_rs::Requirement {
             name,
             extras: requirement.extras,
             version_or_url: Some(VersionOrUrl::Url(requirement.url)),
             marker: requirement.marker,
+            origin: requirement.origin,
         })
     }
 }
