@@ -1,13 +1,13 @@
 use derivative::Derivative;
 
-use pep508_rs::VerbatimUrl;
+use distribution_types::VerbatimParsedUrl;
 use uv_normalize::{ExtraName, PackageName};
 
 use crate::resolver::Urls;
 
 /// A PubGrub-compatible wrapper around a "Python package", with two notable characteristics:
 ///
-/// 1. Includes a [`PubGrubPackage::Root`] variant, to satisfy `PubGrub`'s requirement that a
+/// 1. Includes a [`PubGrubPackage::Root`] variant, to satisfy PubGrub's requirement that a
 ///    resolution starts from a single root.
 /// 2. Uses the same strategy as pip and posy to handle extras: for each extra, we create a virtual
 ///    package (e.g., `black[colorama]`), and mark it as a dependency of the real package (e.g.,
@@ -59,15 +59,33 @@ pub enum PubGrubPackage {
         /// we're going to have a dependency that's provided as a URL, we _need_ to visit the URL
         /// version before the registry version. So we could just error if we visit a URL variant
         /// _after_ a registry variant.
-        Option<VerbatimUrl>,
+        Option<VerbatimParsedUrl>,
     ),
+    /// A proxy package to represent a dependency with an extra (e.g., `black[colorama]`).
+    ///
+    /// For a given package `black`, and an extra `colorama`, we create a virtual package
+    /// with exactly two dependencies: `PubGrubPackage::Package("black", None)` and
+    /// `PubGrubPackage::Package("black", Some("colorama")`. Both dependencies are pinned to the
+    /// same version, and the virtual package is discarded at the end of the resolution process.
+    ///
+    /// The benefit of the proxy package (versus `PubGrubPackage::Package("black", Some("colorama")`
+    /// on its own) is that it enables us to avoid attempting to retrieve metadata for irrelevant
+    /// versions the extra variants by making it clear to PubGrub that the extra variant must match
+    /// the exact same version of the base variant. Without the proxy package, then when provided
+    /// requirements like `black==23.0.1` and `black[colorama]`, PubGrub may attempt to retrieve
+    /// metadata for `black[colorama]` versions other than `23.0.1`.
+    Extra(PackageName, ExtraName, Option<VerbatimParsedUrl>),
 }
 
 impl PubGrubPackage {
     /// Create a [`PubGrubPackage`] from a package name and optional extra name.
     pub(crate) fn from_package(name: PackageName, extra: Option<ExtraName>, urls: &Urls) -> Self {
         let url = urls.get(&name).cloned();
-        Self::Package(name, extra, url)
+        if let Some(extra) = extra {
+            Self::Extra(name, extra, url)
+        } else {
+            Self::Package(name, extra, url)
+        }
     }
 }
 
@@ -94,6 +112,7 @@ impl std::fmt::Display for PubGrubPackage {
             Self::Package(name, Some(extra), ..) => {
                 write!(f, "{name}[{extra}]")
             }
+            Self::Extra(name, extra, ..) => write!(f, "{name}[{extra}]"),
         }
     }
 }
