@@ -1,6 +1,7 @@
 use std::borrow::Cow;
+use std::collections::BTreeSet;
 use std::hash::BuildHasherDefault;
-use std::sync::Arc;
+use std::rc::Rc;
 
 use anyhow::Result;
 use itertools::Itertools;
@@ -13,34 +14,30 @@ use pubgrub::type_aliases::SelectedDependencies;
 use rustc_hash::{FxHashMap, FxHashSet};
 
 use distribution_types::{
-    Dist, DistributionMetadata, IndexUrl, LocalEditable, Name, ResolvedDist, Verbatim, VersionId,
-    VersionOrUrl,
+    Dist, DistributionMetadata, IndexUrl, LocalEditable, Name, ParsedUrlError, Requirement,
+    ResolvedDist, SourceAnnotations, Verbatim, VersionId, VersionOrUrlRef,
 };
 use once_map::OnceMap;
 use pep440_rs::Version;
 use pep508_rs::MarkerEnvironment;
 use pypi_types::HashDigest;
-use uv_distribution::to_precise;
 use uv_normalize::{ExtraName, PackageName};
 
 use crate::dependency_provider::UvDependencyProvider;
 use crate::editables::Editables;
+use crate::lock::{self, Lock, LockError};
 use crate::pins::FilePins;
 use crate::preferences::Preferences;
 use crate::pubgrub::{PubGrubDistribution, PubGrubPackage};
-use crate::redirect::apply_redirect;
+use crate::redirect::url_to_precise;
 use crate::resolver::{InMemoryIndex, MetadataResponse, VersionsResponse};
 use crate::{Manifest, ResolveError};
 
 /// Indicate the style of annotation comments, used to indicate the dependencies that requested each
 /// package.
-#[derive(Debug, Default, Copy, Clone, PartialEq)]
+#[derive(Debug, Default, Copy, Clone, PartialEq, serde::Deserialize)]
+#[serde(deny_unknown_fields, rename_all = "kebab-case")]
 #[cfg_attr(feature = "clap", derive(clap::ValueEnum))]
-#[cfg_attr(feature = "serde", derive(serde::Deserialize))]
-#[cfg_attr(
-    feature = "serde",
-    serde(deny_unknown_fields, rename_all = "kebab-case")
-)]
 #[cfg_attr(feature = "schemars", derive(schemars::JsonSchema))]
 pub enum AnnotationStyle {
     /// Render the annotations on a single, comma-separated line.
@@ -67,13 +64,13 @@ pub struct ResolutionGraph {
 }
 
 impl ResolutionGraph {
-    /// Create a new graph from the resolved `PubGrub` state.
+    /// Create a new graph from the resolved PubGrub state.
     #[allow(clippy::too_many_arguments)]
     pub(crate) fn from_state(
         selection: &SelectedDependencies<UvDependencyProvider>,
         pins: &FilePins,
-        packages: &OnceMap<PackageName, Arc<VersionsResponse>>,
-        distributions: &OnceMap<VersionId, Arc<MetadataResponse>>,
+        packages: &OnceMap<PackageName, Rc<VersionsResponse>>,
+        distributions: &OnceMap<VersionId, Rc<MetadataResponse>>,
         state: &State<UvDependencyProvider>,
         preferences: &Preferences,
         editables: Editables,
@@ -93,7 +90,8 @@ impl ResolutionGraph {
             match package {
                 PubGrubPackage::Package(package_name, None, None) => {
                     // Create the distribution.
-                    let pinned_package = if let Some((editable, _)) = editables.get(package_name) {
+                    let pinned_package = if let Some((editable, _, _)) = editables.get(package_name)
+                    {
                         Dist::from_editable(package_name.clone(), editable.clone())?.into()
                     } else {
                         pins.get(package_name, version)
@@ -126,12 +124,11 @@ impl ResolutionGraph {
                 }
                 PubGrubPackage::Package(package_name, None, Some(url)) => {
                     // Create the distribution.
-                    let pinned_package = if let Some((editable, _)) = editables.get(package_name) {
+                    let pinned_package = if let Some((editable, _, _)) = editables.get(package_name)
+                    {
                         Dist::from_editable(package_name.clone(), editable.clone())?
                     } else {
-                        let url = to_precise(url)
-                            .map_or_else(|| url.clone(), |precise| apply_redirect(url, precise));
-                        Dist::from_url(package_name.clone(), url)?
+                        Dist::from_url(package_name.clone(), url_to_precise(url.clone()))?
                     };
 
                     // Add its hashes to the index, preserving those that were already present in
@@ -159,7 +156,7 @@ impl ResolutionGraph {
                     // Validate that the `extra` exists.
                     let dist = PubGrubDistribution::from_registry(package_name, version);
 
-                    if let Some((editable, metadata)) = editables.get(package_name) {
+                    if let Some((editable, metadata, _)) = editables.get(package_name) {
                         if metadata.provides_extras.contains(extra) {
                             extras
                                 .entry(package_name.clone())
@@ -213,7 +210,7 @@ impl ResolutionGraph {
                     // Validate that the `extra` exists.
                     let dist = PubGrubDistribution::from_url(package_name, url);
 
-                    if let Some((editable, metadata)) = editables.get(package_name) {
+                    if let Some((editable, metadata, _)) = editables.get(package_name) {
                         if metadata.provides_extras.contains(extra) {
                             extras
                                 .entry(package_name.clone())
@@ -249,11 +246,8 @@ impl ResolutionGraph {
                                 .or_insert_with(Vec::new)
                                 .push(extra.clone());
                         } else {
-                            let url = to_precise(url).map_or_else(
-                                || url.clone(),
-                                |precise| apply_redirect(url, precise),
-                            );
-                            let pinned_package = Dist::from_url(package_name.clone(), url)?;
+                            let pinned_package =
+                                Dist::from_url(package_name.clone(), url_to_precise(url.clone()))?;
 
                             diagnostics.push(Diagnostic::MissingExtra {
                                 dist: pinned_package.into(),
@@ -382,7 +376,7 @@ impl ResolutionGraph {
         manifest: &Manifest,
         index: &InMemoryIndex,
         marker_env: &MarkerEnvironment,
-    ) -> pep508_rs::MarkerTree {
+    ) -> Result<pep508_rs::MarkerTree, Box<ParsedUrlError>> {
         use pep508_rs::{
             MarkerExpression, MarkerOperator, MarkerTree, MarkerValue, MarkerValueString,
             MarkerValueVersion,
@@ -437,10 +431,10 @@ impl ResolutionGraph {
         for i in self.petgraph.node_indices() {
             let dist = &self.petgraph[i];
             let version_id = match dist.version_or_url() {
-                VersionOrUrl::Version(version) => {
+                VersionOrUrlRef::Version(version) => {
                     VersionId::from_registry(dist.name().clone(), version.clone())
                 }
-                VersionOrUrl::Url(verbatim_url) => VersionId::from_url(verbatim_url.raw()),
+                VersionOrUrlRef::Url(verbatim_url) => VersionId::from_url(verbatim_url.raw()),
             };
             let res = index
                 .distributions
@@ -452,7 +446,14 @@ impl ResolutionGraph {
                     dist.version_id()
                 )
             };
-            for req in manifest.apply(&archive.metadata.requires_dist) {
+            let requirements: Vec<_> = archive
+                .metadata
+                .requires_dist
+                .iter()
+                .cloned()
+                .map(Requirement::from_pep508)
+                .collect::<Result<_, _>>()?;
+            for req in manifest.apply(requirements.iter()) {
                 let Some(ref marker_tree) = req.marker else {
                     continue;
                 };
@@ -465,7 +466,7 @@ impl ResolutionGraph {
             manifest
                 .editables
                 .iter()
-                .flat_map(|(_, metadata)| &metadata.requires_dist),
+                .flat_map(|(_, _, uv_requirements)| &uv_requirements.dependencies),
         );
         for direct_req in manifest.apply(direct_reqs) {
             let Some(ref marker_tree) = direct_req.marker else {
@@ -498,7 +499,22 @@ impl ResolutionGraph {
             };
             conjuncts.push(MarkerTree::Expression(expr));
         }
-        MarkerTree::And(conjuncts)
+        Ok(MarkerTree::And(conjuncts))
+    }
+
+    pub fn lock(&self) -> Result<Lock, LockError> {
+        let mut locked_dists = vec![];
+        for node_index in self.petgraph.node_indices() {
+            let dist = &self.petgraph[node_index];
+            let mut locked_dist = lock::Distribution::from_resolved_dist(dist)?;
+            for edge in self.petgraph.neighbors(node_index) {
+                let dependency_dist = &self.petgraph[edge];
+                locked_dist.add_dependency(dependency_dist);
+            }
+            locked_dists.push(locked_dist);
+        }
+        let lock = Lock::new(locked_dists)?;
+        Ok(lock)
     }
 }
 
@@ -522,6 +538,8 @@ pub struct DisplayResolutionGraph<'a> {
     /// The style of annotation comments, used to indicate the dependencies that requested each
     /// package.
     annotation_style: AnnotationStyle,
+    /// External sources for each package: requirements, constraints, and overrides.
+    sources: SourceAnnotations,
 }
 
 impl<'a> From<&'a ResolutionGraph> for DisplayResolutionGraph<'a> {
@@ -534,13 +552,14 @@ impl<'a> From<&'a ResolutionGraph> for DisplayResolutionGraph<'a> {
             true,
             false,
             AnnotationStyle::default(),
+            SourceAnnotations::default(),
         )
     }
 }
 
 impl<'a> DisplayResolutionGraph<'a> {
     /// Create a new [`DisplayResolutionGraph`] for the given graph.
-    #[allow(clippy::fn_params_excessive_bools)]
+    #[allow(clippy::fn_params_excessive_bools, clippy::too_many_arguments)]
     pub fn new(
         underlying: &'a ResolutionGraph,
         no_emit_packages: &'a [PackageName],
@@ -549,6 +568,7 @@ impl<'a> DisplayResolutionGraph<'a> {
         include_annotations: bool,
         include_index_annotation: bool,
         annotation_style: AnnotationStyle,
+        sources: SourceAnnotations,
     ) -> DisplayResolutionGraph<'a> {
         Self {
             resolution: underlying,
@@ -558,6 +578,7 @@ impl<'a> DisplayResolutionGraph<'a> {
             include_annotations,
             include_index_annotation,
             annotation_style,
+            sources,
         }
     }
 }
@@ -639,7 +660,7 @@ impl std::fmt::Display for DisplayResolutionGraph<'_> {
                     return None;
                 }
 
-                let node = if let Some((editable, _)) = self.resolution.editables.get(name) {
+                let node = if let Some((editable, _, _)) = self.resolution.editables.get(name) {
                     Node::Editable(name, editable)
                 } else if self.include_extras {
                     Node::Distribution(
@@ -698,13 +719,23 @@ impl std::fmt::Display for DisplayResolutionGraph<'_> {
                     .collect::<Vec<_>>();
                 edges.sort_unstable_by_key(|package| package.name());
 
+                // Include all external sources (e.g., requirements files).
+                let default = BTreeSet::default();
+                let source = match node {
+                    Node::Editable(_, editable) => {
+                        self.sources.get_editable(&editable.url).unwrap_or(&default)
+                    }
+                    Node::Distribution(name, _, _) => self.sources.get(name).unwrap_or(&default),
+                };
+
                 match self.annotation_style {
                     AnnotationStyle::Line => {
                         if !edges.is_empty() {
                             let separator = if has_hashes { "\n    " } else { "  " };
                             let deps = edges
                                 .into_iter()
-                                .map(|dependency| dependency.name().to_string())
+                                .map(|dependency| format!("{}", dependency.name()))
+                                .chain(source.iter().map(std::string::ToString::to_string))
                                 .collect::<Vec<_>>()
                                 .join(", ");
                             let comment = format!("# via {deps}").green().to_string();
@@ -712,17 +743,30 @@ impl std::fmt::Display for DisplayResolutionGraph<'_> {
                         }
                     }
                     AnnotationStyle::Split => match edges.as_slice() {
-                        [] => {}
-                        [edge] => {
+                        [] if source.is_empty() => {}
+                        [] if source.len() == 1 => {
+                            let separator = "\n";
+                            let comment = format!("    # via {}", source.iter().next().unwrap())
+                                .green()
+                                .to_string();
+                            annotation = Some((separator, comment));
+                        }
+                        [edge] if source.is_empty() => {
                             let separator = "\n";
                             let comment = format!("    # via {}", edge.name()).green().to_string();
                             annotation = Some((separator, comment));
                         }
                         edges => {
                             let separator = "\n";
-                            let deps = edges
+                            let deps = source
                                 .iter()
-                                .map(|dependency| format!("    #   {}", dependency.name()))
+                                .map(std::string::ToString::to_string)
+                                .chain(
+                                    edges
+                                        .iter()
+                                        .map(|dependency| format!("{}", dependency.name())),
+                                )
+                                .map(|name| format!("    #   {name}"))
                                 .collect::<Vec<_>>()
                                 .join("\n");
                             let comment = format!("    # via\n{deps}").green().to_string();
