@@ -29,14 +29,13 @@ use pep508_rs::MarkerEnvironment;
 use platform_tags::Tags;
 use pypi_types::Metadata23;
 pub(crate) use urls::Urls;
-use uv_client::RegistryClient;
 use uv_configuration::{Constraints, Overrides};
 use uv_distribution::{ArchiveMetadata, DistributionDatabase};
-use uv_interpreter::Interpreter;
 use uv_normalize::PackageName;
 use uv_types::{BuildContext, HashStrategy, InstalledPackagesProvider};
 
 use crate::candidate_selector::{CandidateDist, CandidateSelector};
+use crate::dependency_provider::UvDependencyProvider;
 use crate::editables::Editables;
 use crate::error::ResolveError;
 use crate::manifest::Manifest;
@@ -44,7 +43,7 @@ use crate::pins::FilePins;
 use crate::preferences::Preferences;
 use crate::pubgrub::{
     PubGrubDependencies, PubGrubDistribution, PubGrubPackage, PubGrubPriorities, PubGrubPython,
-    PubGrubSpecifier,
+    PubGrubRequirement, PubGrubSpecifier,
 };
 use crate::python_requirement::PythonRequirement;
 use crate::resolution::ResolutionGraph;
@@ -188,8 +187,9 @@ pub struct Resolver<'a, Provider: ResolverProvider, InstalledPackages: Installed
     locals: Locals,
     dependency_mode: DependencyMode,
     hasher: &'a HashStrategy,
-    markers: &'a MarkerEnvironment,
-    python_requirement: PythonRequirement,
+    /// When not set, the resolver is in "universal" mode.
+    markers: Option<&'a MarkerEnvironment>,
+    python_requirement: &'a PythonRequirement,
     selector: CandidateSelector,
     index: &'a InMemoryIndex,
     installed_packages: &'a InstalledPackages,
@@ -209,38 +209,53 @@ impl<'a, Context: BuildContext, InstalledPackages: InstalledPackagesProvider>
     /// Initialize a new resolver using the default backend doing real requests.
     ///
     /// Reads the flat index entries.
+    ///
+    /// # Marker environment
+    ///
+    /// The marker environment is optional.
+    ///
+    /// When a marker environment is not provided, the resolver is said to be
+    /// in "universal" mode. When in universal mode, the resolution produced
+    /// may contain multiple versions of the same package. And thus, in order
+    /// to use the resulting resolution, there must be a "universal"-aware
+    /// reader of the resolution that knows to exclude distributions that can't
+    /// be used in the current environment.
+    ///
+    /// When a marker environment is provided, the resolver is in
+    /// "non-universal" mode, which corresponds to standard `pip` behavior that
+    /// works only for a specific marker environment.
     #[allow(clippy::too_many_arguments)]
     pub fn new(
         manifest: Manifest,
         options: Options,
-        markers: &'a MarkerEnvironment,
-        interpreter: &'a Interpreter,
+        python_requirement: &'a PythonRequirement,
+        markers: Option<&'a MarkerEnvironment>,
         tags: &'a Tags,
-        client: &'a RegistryClient,
         flat_index: &'a FlatIndex,
         index: &'a InMemoryIndex,
         hasher: &'a HashStrategy,
         build_context: &'a Context,
         installed_packages: &'a InstalledPackages,
+        database: DistributionDatabase<'a, Context>,
     ) -> Result<Self, ResolveError> {
         let provider = DefaultResolverProvider::new(
-            client,
-            DistributionDatabase::new(client, build_context),
+            database,
             flat_index,
             tags,
-            PythonRequirement::new(interpreter, markers),
+            python_requirement.clone(),
             AllowedYanks::from_manifest(&manifest, markers, options.dependency_mode),
             hasher,
             options.exclude_newer,
             build_context.no_binary(),
             build_context.no_build(),
         );
+
         Self::new_custom_io(
             manifest,
             options,
             hasher,
             markers,
-            PythonRequirement::new(interpreter, markers),
+            python_requirement,
             index,
             provider,
             installed_packages,
@@ -257,8 +272,8 @@ impl<'a, Provider: ResolverProvider, InstalledPackages: InstalledPackagesProvide
         manifest: Manifest,
         options: Options,
         hasher: &'a HashStrategy,
-        markers: &'a MarkerEnvironment,
-        python_requirement: PythonRequirement,
+        markers: Option<&'a MarkerEnvironment>,
+        python_requirement: &'a PythonRequirement,
         index: &'a InMemoryIndex,
         provider: Provider,
         installed_packages: &'a InstalledPackages,
@@ -323,12 +338,12 @@ impl<'a, Provider: ResolverProvider, InstalledPackages: InstalledPackagesProvide
                 Err(if let ResolveError::NoSolution(err) = err {
                     ResolveError::NoSolution(
                         err.with_available_versions(
-                            &self.python_requirement,
+                            self.python_requirement,
                             &self.visited,
                             &self.index.packages,
                         )
                         .with_selector(self.selector.clone())
-                        .with_python_requirement(&self.python_requirement)
+                        .with_python_requirement(self.python_requirement)
                         .with_index_locations(self.provider.index_locations())
                         .with_unavailable_packages(&self.unavailable_packages)
                         .with_incomplete_packages(&self.incomplete_packages),
@@ -348,16 +363,13 @@ impl<'a, Provider: ResolverProvider, InstalledPackages: InstalledPackagesProvide
     ) -> Result<ResolutionGraph, ResolveError> {
         let root = PubGrubPackage::Root(self.project.clone());
         let mut prefetcher = BatchPrefetcher::default();
-
-        // Keep track of the packages for which we've requested metadata.
-        let mut pins = FilePins::default();
-        let mut priorities = PubGrubPriorities::default();
-
-        // Start the solve.
-        let mut state = State::init(root.clone(), MIN_VERSION.clone());
-        let mut added_dependencies: FxHashMap<PubGrubPackage, FxHashSet<Version>> =
-            FxHashMap::default();
-        let mut next = root;
+        let mut state = ResolverState {
+            pubgrub: State::init(root.clone(), MIN_VERSION.clone()),
+            next: root,
+            pins: FilePins::default(),
+            priorities: PubGrubPriorities::default(),
+            added_dependencies: FxHashMap::default(),
+        };
 
         debug!(
             "Solving with target Python version {}",
@@ -366,49 +378,54 @@ impl<'a, Provider: ResolverProvider, InstalledPackages: InstalledPackagesProvide
 
         loop {
             // Run unit propagation.
-            state.unit_propagation(next)?;
+            state.pubgrub.unit_propagation(state.next)?;
 
             // Pre-visit all candidate packages, to allow metadata to be fetched in parallel. If
             // the dependency mode is direct, we only need to visit the root package.
             if self.dependency_mode.is_transitive() {
-                Self::pre_visit(state.partial_solution.prioritized_packages(), &request_sink)
-                    .await?;
+                Self::pre_visit(
+                    state.pubgrub.partial_solution.prioritized_packages(),
+                    &request_sink,
+                )
+                .await?;
             }
 
             // Choose a package version.
             let Some(highest_priority_pkg) = state
+                .pubgrub
                 .partial_solution
-                .pick_highest_priority_pkg(|package, _range| priorities.get(package))
+                .pick_highest_priority_pkg(|package, _range| state.priorities.get(package))
             else {
                 if enabled!(Level::DEBUG) {
                     prefetcher.log_tried_versions();
                 }
-                let selection = state.partial_solution.extract_solution();
+                let selection = state.pubgrub.partial_solution.extract_solution();
                 return ResolutionGraph::from_state(
                     &selection,
-                    &pins,
+                    &state.pins,
                     &self.index.packages,
                     &self.index.distributions,
-                    &state,
+                    &state.pubgrub,
                     &self.preferences,
                     self.editables.clone(),
                 );
             };
-            next = highest_priority_pkg;
+            state.next = highest_priority_pkg;
 
-            prefetcher.version_tried(next.clone());
+            prefetcher.version_tried(state.next.clone());
 
             let term_intersection = state
+                .pubgrub
                 .partial_solution
-                .term_intersection_for_package(&next)
+                .term_intersection_for_package(&state.next)
                 .ok_or_else(|| {
                     PubGrubError::Failure("a package was chosen but we don't have a term.".into())
                 })?;
             let decision = self
                 .choose_version(
-                    &next,
+                    &state.next,
                     term_intersection.unwrap_positive(),
-                    &mut pins,
+                    &mut state.pins,
                     &request_sink,
                 )
                 .await?;
@@ -416,29 +433,34 @@ impl<'a, Provider: ResolverProvider, InstalledPackages: InstalledPackagesProvide
             // Pick the next compatible version.
             let version = match decision {
                 None => {
-                    debug!("No compatible version found for: {next}");
+                    debug!("No compatible version found for: {next}", next = state.next);
 
                     let term_intersection = state
+                        .pubgrub
                         .partial_solution
-                        .term_intersection_for_package(&next)
+                        .term_intersection_for_package(&state.next)
                         .expect("a package was chosen but we don't have a term.");
 
                     // Check if the decision was due to the package being unavailable
-                    if let PubGrubPackage::Package(ref package_name, _, _) = next {
+                    if let PubGrubPackage::Package(ref package_name, _, _) = state.next {
                         if let Some(entry) = self.unavailable_packages.borrow().get(package_name) {
-                            state.add_incompatibility(Incompatibility::custom_term(
-                                next.clone(),
-                                term_intersection.clone(),
-                                UnavailableReason::Package(entry.clone()),
-                            ));
+                            state
+                                .pubgrub
+                                .add_incompatibility(Incompatibility::custom_term(
+                                    state.next.clone(),
+                                    term_intersection.clone(),
+                                    UnavailableReason::Package(entry.clone()),
+                                ));
                             continue;
                         }
                     }
 
-                    state.add_incompatibility(Incompatibility::no_versions(
-                        next.clone(),
-                        term_intersection.clone(),
-                    ));
+                    state
+                        .pubgrub
+                        .add_incompatibility(Incompatibility::no_versions(
+                            state.next.clone(),
+                            term_intersection.clone(),
+                        ));
                     continue;
                 }
                 Some(version) => version,
@@ -463,29 +485,36 @@ impl<'a, Provider: ResolverProvider, InstalledPackages: InstalledPackagesProvide
                                 range.intersection(&specifier.into())
                             })?;
 
-                        let package = &next;
+                        let package = &state.next;
                         for kind in [PubGrubPython::Installed, PubGrubPython::Target] {
-                            state.add_incompatibility(Incompatibility::from_dependency(
-                                package.clone(),
-                                Range::singleton(version.clone()),
-                                (PubGrubPackage::Python(kind), python_version.clone()),
-                            ));
+                            state
+                                .pubgrub
+                                .add_incompatibility(Incompatibility::from_dependency(
+                                    package.clone(),
+                                    Range::singleton(version.clone()),
+                                    (PubGrubPackage::Python(kind), python_version.clone()),
+                                ));
                         }
-                        state.partial_solution.add_decision(next.clone(), version);
+                        state
+                            .pubgrub
+                            .partial_solution
+                            .add_decision(state.next.clone(), version);
                         continue;
                     };
-                    state.add_incompatibility(Incompatibility::custom_version(
-                        next.clone(),
-                        version.clone(),
-                        UnavailableReason::Version(reason),
-                    ));
+                    state
+                        .pubgrub
+                        .add_incompatibility(Incompatibility::custom_version(
+                            state.next.clone(),
+                            version.clone(),
+                            UnavailableReason::Version(reason),
+                        ));
                     continue;
                 }
             };
 
             prefetcher
                 .prefetch_batches(
-                    &next,
+                    &state.next,
                     &version,
                     term_intersection.unwrap_positive(),
                     &request_sink,
@@ -494,25 +523,28 @@ impl<'a, Provider: ResolverProvider, InstalledPackages: InstalledPackagesProvide
                 )
                 .await?;
 
-            self.on_progress(&next, &version);
+            self.on_progress(&state.next, &version);
 
-            if added_dependencies
-                .entry(next.clone())
+            if state
+                .added_dependencies
+                .entry(state.next.clone())
                 .or_default()
                 .insert(version.clone())
             {
                 // Retrieve that package dependencies.
-                let package = &next;
+                let package = &state.next;
                 let dependencies = match self
-                    .get_dependencies(package, &version, &mut priorities, &request_sink)
+                    .get_dependencies(package, &version, &mut state.priorities, &request_sink)
                     .await?
                 {
                     Dependencies::Unavailable(reason) => {
-                        state.add_incompatibility(Incompatibility::custom_version(
-                            package.clone(),
-                            version.clone(),
-                            UnavailableReason::Version(reason),
-                        ));
+                        state
+                            .pubgrub
+                            .add_incompatibility(Incompatibility::custom_version(
+                                package.clone(),
+                                version.clone(),
+                                UnavailableReason::Version(reason),
+                            ));
                         continue;
                     }
                     Dependencies::Available(constraints)
@@ -533,22 +565,25 @@ impl<'a, Provider: ResolverProvider, InstalledPackages: InstalledPackagesProvide
                 };
 
                 // Add that package and version if the dependencies are not problematic.
-                let dep_incompats = state.add_incompatibility_from_dependencies(
+                let dep_incompats = state.pubgrub.add_incompatibility_from_dependencies(
                     package.clone(),
                     version.clone(),
                     dependencies,
                 );
 
-                state.partial_solution.add_version(
+                state.pubgrub.partial_solution.add_version(
                     package.clone(),
                     version,
                     dep_incompats,
-                    &state.incompatibility_store,
+                    &state.pubgrub.incompatibility_store,
                 );
             } else {
                 // `dep_incompats` are already in `incompatibilities` so we know there are not satisfied
                 // terms and can add the decision directly.
-                state.partial_solution.add_decision(next.clone(), version);
+                state
+                    .pubgrub
+                    .partial_solution
+                    .add_decision(state.next.clone(), version);
             }
         }
     }
@@ -577,10 +612,15 @@ impl<'a, Provider: ResolverProvider, InstalledPackages: InstalledPackagesProvide
             }
             PubGrubPackage::Package(name, _extra, Some(url)) => {
                 // Verify that the package is allowed under the hash-checking policy.
-                if !self.hasher.allows_url(url) {
+                if !self.hasher.allows_url(&url.verbatim) {
                     return Err(ResolveError::UnhashedPackage(name.clone()));
                 }
 
+                // If the package is an editable, we don't need to fetch metadata.
+                if self.editables.contains(name) {
+                    return Ok(());
+                }
+
                 // Emit a request to fetch the metadata for this distribution.
                 let dist = Dist::from_url(name.clone(), url.clone())?;
                 if self.index.distributions.register(dist.version_id()) {
@@ -645,7 +685,10 @@ impl<'a, Provider: ResolverProvider, InstalledPackages: InstalledPackagesProvide
 
             PubGrubPackage::Extra(package_name, _, Some(url))
             | PubGrubPackage::Package(package_name, _, Some(url)) => {
-                debug!("Searching for a compatible version of {package} @ {url} ({range})");
+                debug!(
+                    "Searching for a compatible version of {package} @ {} ({range})",
+                    url.verbatim
+                );
 
                 // If the dist is an editable, return the version from the editable metadata.
                 if let Some((_local, metadata, _)) = self.editables.get(package_name) {
@@ -843,7 +886,7 @@ impl<'a, Provider: ResolverProvider, InstalledPackages: InstalledPackagesProvide
         match package {
             PubGrubPackage::Root(_) => {
                 // Add the root requirements.
-                let constraints = PubGrubDependencies::from_requirements(
+                let dependencies = PubGrubDependencies::from_requirements(
                     &self.requirements,
                     &self.constraints,
                     &self.overrides,
@@ -854,8 +897,8 @@ impl<'a, Provider: ResolverProvider, InstalledPackages: InstalledPackagesProvide
                     self.markers,
                 );
 
-                let mut constraints = match constraints {
-                    Ok(constraints) => constraints,
+                let mut dependencies = match dependencies {
+                    Ok(dependencies) => dependencies,
                     Err(err) => {
                         return Ok(Dependencies::Unavailable(
                             UnavailableVersion::ResolverError(uncapitalize(err.to_string())),
@@ -863,7 +906,7 @@ impl<'a, Provider: ResolverProvider, InstalledPackages: InstalledPackagesProvide
                     }
                 };
 
-                for (package, version) in constraints.iter() {
+                for (package, version) in dependencies.iter() {
                     debug!("Adding direct dependency: {package}{version}");
 
                     // Update the package priorities.
@@ -883,11 +926,11 @@ impl<'a, Provider: ResolverProvider, InstalledPackages: InstalledPackagesProvide
                     priorities.insert(&package, &version);
 
                     // Add the editable as a direct dependency.
-                    constraints.push(package, version);
+                    dependencies.push(package, version);
 
                     // Add a dependency on each extra.
                     for extra in &editable.extras {
-                        constraints.push(
+                        dependencies.push(
                             PubGrubPackage::from_package(
                                 metadata.name.clone(),
                                 Some(extra.clone()),
@@ -896,9 +939,22 @@ impl<'a, Provider: ResolverProvider, InstalledPackages: InstalledPackagesProvide
                             Range::singleton(metadata.version.clone()),
                         );
                     }
+
+                    // Add any constraints.
+                    for constraint in self.constraints.get(&metadata.name).into_iter().flatten() {
+                        if constraint.evaluate_markers(self.markers, &[]) {
+                            let PubGrubRequirement { package, version } =
+                                PubGrubRequirement::from_constraint(
+                                    constraint,
+                                    &self.urls,
+                                    &self.locals,
+                                )?;
+                            dependencies.push(package, version);
+                        }
+                    }
                 }
 
-                Ok(Dependencies::Available(constraints.into()))
+                Ok(Dependencies::Available(dependencies.into()))
             }
 
             PubGrubPackage::Python(_) => Ok(Dependencies::Available(Vec::default())),
@@ -908,7 +964,7 @@ impl<'a, Provider: ResolverProvider, InstalledPackages: InstalledPackagesProvide
                 if self.dependency_mode.is_direct() {
                     // If an extra is provided, wait for the metadata to be available, since it's
                     // still required for reporting diagnostics.
-                    if extra.is_some() && self.editables.get(package_name).is_none() {
+                    if extra.is_some() && !self.editables.contains(package_name) {
                         // Determine the distribution to lookup.
                         let dist = match url {
                             Some(url) => PubGrubDistribution::from_url(package_name, url),
@@ -936,7 +992,7 @@ impl<'a, Provider: ResolverProvider, InstalledPackages: InstalledPackagesProvide
                         .cloned()
                         .map(Requirement::from_pep508)
                         .collect::<Result<_, _>>()?;
-                    let constraints = PubGrubDependencies::from_requirements(
+                    let dependencies = PubGrubDependencies::from_requirements(
                         &requirements,
                         &self.constraints,
                         &self.overrides,
@@ -947,7 +1003,7 @@ impl<'a, Provider: ResolverProvider, InstalledPackages: InstalledPackagesProvide
                         self.markers,
                     )?;
 
-                    for (dep_package, dep_version) in constraints.iter() {
+                    for (dep_package, dep_version) in dependencies.iter() {
                         debug!("Adding transitive dependency for {package}=={version}: {dep_package}{dep_version}");
 
                         // Update the package priorities.
@@ -957,7 +1013,7 @@ impl<'a, Provider: ResolverProvider, InstalledPackages: InstalledPackagesProvide
                         self.visit_package(dep_package, request_sink).await?;
                     }
 
-                    return Ok(Dependencies::Available(constraints.into()));
+                    return Ok(Dependencies::Available(dependencies.into()));
                 }
 
                 // Determine the distribution to lookup.
@@ -1060,7 +1116,7 @@ impl<'a, Provider: ResolverProvider, InstalledPackages: InstalledPackagesProvide
                     .cloned()
                     .map(Requirement::from_pep508)
                     .collect::<Result<_, _>>()?;
-                let constraints = PubGrubDependencies::from_requirements(
+                let dependencies = PubGrubDependencies::from_requirements(
                     &requirements,
                     &self.constraints,
                     &self.overrides,
@@ -1071,7 +1127,7 @@ impl<'a, Provider: ResolverProvider, InstalledPackages: InstalledPackagesProvide
                     self.markers,
                 )?;
 
-                for (dep_package, dep_version) in constraints.iter() {
+                for (dep_package, dep_version) in dependencies.iter() {
                     debug!("Adding transitive dependency for {package}=={version}: {dep_package}{dep_version}");
 
                     // Update the package priorities.
@@ -1081,7 +1137,7 @@ impl<'a, Provider: ResolverProvider, InstalledPackages: InstalledPackagesProvide
                     self.visit_package(dep_package, request_sink).await?;
                 }
 
-                Ok(Dependencies::Available(constraints.into()))
+                Ok(Dependencies::Available(dependencies.into()))
             }
 
             // Add a dependency on both the extra and base package.
@@ -1105,7 +1161,10 @@ impl<'a, Provider: ResolverProvider, InstalledPackages: InstalledPackagesProvide
     ) -> Result<(), ResolveError> {
         let mut response_stream = ReceiverStream::new(request_stream)
             .map(|request| self.process_request(request).boxed_local())
-            .buffer_unordered(50);
+            // Allow as many futures as possible to start in the background.
+            // Backpressure is provided by at a more granular level by `DistributionDatabase`
+            // and `SourceDispatch`, as well as the bounded request channel.
+            .buffer_unordered(usize::MAX);
 
         while let Some(response) = response_stream.next().await {
             match response? {
@@ -1186,10 +1245,13 @@ impl<'a, Provider: ResolverProvider, InstalledPackages: InstalledPackagesProvide
                     .boxed_local()
                     .await
                     .map_err(|err| match dist.clone() {
-                        Dist::Built(BuiltDist::Path(built_dist)) => {
+                        Dist::Built(built_dist @ BuiltDist::Path(_)) => {
                             ResolveError::Read(Box::new(built_dist), err)
                         }
-                        Dist::Source(SourceDist::Path(source_dist)) => {
+                        Dist::Source(source_dist @ SourceDist::Path(_)) => {
+                            ResolveError::Build(Box::new(source_dist), err)
+                        }
+                        Dist::Source(source_dist @ SourceDist::Directory(_)) => {
                             ResolveError::Build(Box::new(source_dist), err)
                         }
                         Dist::Built(built_dist) => ResolveError::Fetch(Box::new(built_dist), err),
@@ -1273,10 +1335,13 @@ impl<'a, Provider: ResolverProvider, InstalledPackages: InstalledPackagesProvide
                                 .boxed_local()
                                 .await
                                 .map_err(|err| match dist.clone() {
-                                    Dist::Built(BuiltDist::Path(built_dist)) => {
+                                    Dist::Built(built_dist @ BuiltDist::Path(_)) => {
                                         ResolveError::Read(Box::new(built_dist), err)
                                     }
-                                    Dist::Source(SourceDist::Path(source_dist)) => {
+                                    Dist::Source(source_dist @ SourceDist::Path(_)) => {
+                                        ResolveError::Build(Box::new(source_dist), err)
+                                    }
+                                    Dist::Source(source_dist @ SourceDist::Directory(_)) => {
                                         ResolveError::Build(Box::new(source_dist), err)
                                     }
                                     Dist::Built(built_dist) => {
@@ -1311,7 +1376,7 @@ impl<'a, Provider: ResolverProvider, InstalledPackages: InstalledPackagesProvide
                 PubGrubPackage::Python(_) => {}
                 PubGrubPackage::Extra(_, _, _) => {}
                 PubGrubPackage::Package(package_name, _extra, Some(url)) => {
-                    reporter.on_progress(package_name, &VersionOrUrlRef::Url(url));
+                    reporter.on_progress(package_name, &VersionOrUrlRef::Url(&url.verbatim));
                 }
                 PubGrubPackage::Package(package_name, _extra, None) => {
                     reporter.on_progress(package_name, &VersionOrUrlRef::Version(version));
@@ -1327,6 +1392,40 @@ impl<'a, Provider: ResolverProvider, InstalledPackages: InstalledPackagesProvide
     }
 }
 
+/// State that is used during unit propagation in the resolver.
+#[derive(Clone)]
+struct ResolverState {
+    /// The internal state used by the resolver.
+    ///
+    /// Note that not all parts of this state are strictly internal. For
+    /// example, the edges in the dependency graph generated as part of the
+    /// output of resolution are derived from the "incompatibilities" tracked
+    /// in this state. We also ultimately retrieve the final set of version
+    /// assignments (to packages) from this state's "partial solution."
+    pubgrub: State<UvDependencyProvider>,
+    /// The next package on which to run unit propagation.
+    next: PubGrubPackage,
+    /// The set of pinned versions we accrue throughout resolution.
+    ///
+    /// The key of this map is a package name, and each package name maps to
+    /// a set of versions for that package. Each version in turn is mapped
+    /// to a single `ResolvedDist`. That `ResolvedDist` represents, at time
+    /// of writing (2024/05/09), at most one wheel. The idea here is that
+    /// `FilePins` tracks precisely which wheel was selected during resolution.
+    /// After resolution is finished, this maps is consulted in order to select
+    /// the wheel chosen during resolution.
+    pins: FilePins,
+    /// When dependencies for a package are retrieved, this map of priorities
+    /// is updated based on how each dependency was specified. Certain types
+    /// of dependencies have more "priority" than others (like direct URL
+    /// dependencies). These priorities help determine which package to
+    /// consider next during resolution.
+    priorities: PubGrubPriorities,
+    /// This keeps track of the set of versions for each package that we've
+    /// already visited during resolution. This avoids doing redundant work.
+    added_dependencies: FxHashMap<PubGrubPackage, FxHashSet<Version>>,
+}
+
 /// Fetch the metadata for an item
 #[derive(Debug)]
 #[allow(clippy::large_enum_variant)]
