@@ -10,24 +10,32 @@ use anyhow::Result;
 use chrono::{DateTime, Utc};
 use once_cell::sync::Lazy;
 
-use distribution_types::{IndexLocations, Resolution, SourceDist};
-use pep508_rs::{MarkerEnvironment, Requirement, StringVersion};
+use distribution_types::{IndexLocations, Requirement, Resolution, SourceDist};
+use pep508_rs::{MarkerEnvironment, MarkerEnvironmentBuilder};
 use platform_tags::{Arch, Os, Platform, Tags};
 use uv_cache::Cache;
-use uv_client::{FlatIndex, RegistryClientBuilder};
-use uv_configuration::{BuildKind, Constraints, NoBinary, NoBuild, Overrides, SetupPyStrategy};
+use uv_client::RegistryClientBuilder;
+use uv_configuration::{
+    BuildKind, Concurrency, Constraints, NoBinary, NoBuild, Overrides, SetupPyStrategy,
+};
+use uv_distribution::DistributionDatabase;
 use uv_interpreter::{find_default_python, Interpreter, PythonEnvironment};
 use uv_resolver::{
-    DisplayResolutionGraph, Exclusions, InMemoryIndex, Manifest, Options, OptionsBuilder,
-    PreReleaseMode, Preference, ResolutionGraph, ResolutionMode, Resolver,
+    DisplayResolutionGraph, ExcludeNewer, Exclusions, FlatIndex, InMemoryIndex, Manifest, Options,
+    OptionsBuilder, PreReleaseMode, Preference, PythonRequirement, ResolutionGraph, ResolutionMode,
+    Resolver,
+};
+use uv_types::{
+    BuildContext, BuildIsolation, EmptyInstalledPackages, HashStrategy, SourceBuildTrait,
 };
-use uv_types::{BuildContext, BuildIsolation, EmptyInstalledPackages, SourceBuildTrait};
 
 // Exclude any packages uploaded after this date.
-static EXCLUDE_NEWER: Lazy<DateTime<Utc>> = Lazy::new(|| {
-    DateTime::parse_from_rfc3339("2023-11-18T12:00:00Z")
-        .unwrap()
-        .with_timezone(&Utc)
+static EXCLUDE_NEWER: Lazy<ExcludeNewer> = Lazy::new(|| {
+    ExcludeNewer::from(
+        DateTime::parse_from_rfc3339("2023-11-18T12:00:00Z")
+            .unwrap()
+            .with_timezone(&Utc),
+    )
 });
 
 struct DummyContext {
@@ -122,19 +130,23 @@ async fn resolve(
     let real_interpreter =
         find_default_python(&Cache::temp().unwrap()).expect("Expected a python to be installed");
     let interpreter = Interpreter::artificial(real_interpreter.platform().clone(), markers.clone());
+    let python_requirement = PythonRequirement::from_marker_environment(&interpreter, markers);
     let build_context = DummyContext::new(Cache::temp()?, interpreter.clone());
+    let hashes = HashStrategy::None;
     let installed_packages = EmptyInstalledPackages;
+    let concurrency = Concurrency::default();
     let resolver = Resolver::new(
         manifest,
         options,
-        markers,
-        &interpreter,
+        &python_requirement,
+        Some(markers),
         tags,
-        &client,
         &flat_index,
         &index,
+        &hashes,
         &build_context,
         &installed_packages,
+        DistributionDatabase::new(&client, &build_context, concurrency.downloads),
     )?;
     Ok(resolver.resolve().await?)
 }
@@ -148,7 +160,10 @@ macro_rules! assert_snapshot {
 
 #[tokio::test]
 async fn black() -> Result<()> {
-    let manifest = Manifest::simple(vec![Requirement::from_str("black<=23.9.1").unwrap()]);
+    let manifest = Manifest::simple(vec![Requirement::from_pep508(
+        pep508_rs::Requirement::from_str("black<=23.9.1").unwrap(),
+    )
+    .unwrap()]);
     let options = OptionsBuilder::new()
         .exclude_newer(Some(*EXCLUDE_NEWER))
         .build();
@@ -174,9 +189,10 @@ async fn black() -> Result<()> {
 
 #[tokio::test]
 async fn black_colorama() -> Result<()> {
-    let manifest = Manifest::simple(vec![
-        Requirement::from_str("black[colorama]<=23.9.1").unwrap()
-    ]);
+    let manifest = Manifest::simple(vec![Requirement::from_pep508(
+        pep508_rs::Requirement::from_str("black[colorama]<=23.9.1").unwrap(),
+    )
+    .unwrap()]);
     let options = OptionsBuilder::new()
         .exclude_newer(Some(*EXCLUDE_NEWER))
         .build();
@@ -205,9 +221,10 @@ async fn black_colorama() -> Result<()> {
 /// Resolve Black with an invalid extra. The resolver should ignore the extra.
 #[tokio::test]
 async fn black_tensorboard() -> Result<()> {
-    let manifest = Manifest::simple(vec![
-        Requirement::from_str("black[tensorboard]<=23.9.1").unwrap()
-    ]);
+    let manifest = Manifest::simple(vec![Requirement::from_pep508(
+        pep508_rs::Requirement::from_str("black[tensorboard]<=23.9.1").unwrap(),
+    )
+    .unwrap()]);
     let options = OptionsBuilder::new()
         .exclude_newer(Some(*EXCLUDE_NEWER))
         .build();
@@ -233,7 +250,10 @@ async fn black_tensorboard() -> Result<()> {
 
 #[tokio::test]
 async fn black_python_310() -> Result<()> {
-    let manifest = Manifest::simple(vec![Requirement::from_str("black<=23.9.1").unwrap()]);
+    let manifest = Manifest::simple(vec![Requirement::from_pep508(
+        pep508_rs::Requirement::from_str("black<=23.9.1").unwrap(),
+    )
+    .unwrap()]);
     let options = OptionsBuilder::new()
         .exclude_newer(Some(*EXCLUDE_NEWER))
         .build();
@@ -266,10 +286,14 @@ async fn black_python_310() -> Result<()> {
 #[tokio::test]
 async fn black_mypy_extensions() -> Result<()> {
     let manifest = Manifest::new(
-        vec![Requirement::from_str("black<=23.9.1").unwrap()],
-        Constraints::from_requirements(vec![
-            Requirement::from_str("mypy-extensions<0.4.4").unwrap()
-        ]),
+        vec![
+            Requirement::from_pep508(pep508_rs::Requirement::from_str("black<=23.9.1").unwrap())
+                .unwrap(),
+        ],
+        Constraints::from_requirements(vec![Requirement::from_pep508(
+            pep508_rs::Requirement::from_str("mypy-extensions<0.4.4").unwrap(),
+        )
+        .unwrap()]),
         Overrides::default(),
         vec![],
         None,
@@ -305,10 +329,14 @@ async fn black_mypy_extensions() -> Result<()> {
 #[tokio::test]
 async fn black_mypy_extensions_extra() -> Result<()> {
     let manifest = Manifest::new(
-        vec![Requirement::from_str("black<=23.9.1").unwrap()],
-        Constraints::from_requirements(vec![
-            Requirement::from_str("mypy-extensions[extra]<0.4.4").unwrap()
-        ]),
+        vec![
+            Requirement::from_pep508(pep508_rs::Requirement::from_str("black<=23.9.1").unwrap())
+                .unwrap(),
+        ],
+        Constraints::from_requirements(vec![Requirement::from_pep508(
+            pep508_rs::Requirement::from_str("mypy-extensions[extra]<0.4.4").unwrap(),
+        )
+        .unwrap()]),
         Overrides::default(),
         vec![],
         None,
@@ -344,8 +372,14 @@ async fn black_mypy_extensions_extra() -> Result<()> {
 #[tokio::test]
 async fn black_flake8() -> Result<()> {
     let manifest = Manifest::new(
-        vec![Requirement::from_str("black<=23.9.1").unwrap()],
-        Constraints::from_requirements(vec![Requirement::from_str("flake8<1").unwrap()]),
+        vec![
+            Requirement::from_pep508(pep508_rs::Requirement::from_str("black<=23.9.1").unwrap())
+                .unwrap(),
+        ],
+        Constraints::from_requirements(vec![Requirement::from_pep508(
+            pep508_rs::Requirement::from_str("flake8<1").unwrap(),
+        )
+        .unwrap()]),
         Overrides::default(),
         vec![],
         None,
@@ -378,7 +412,10 @@ async fn black_flake8() -> Result<()> {
 
 #[tokio::test]
 async fn black_lowest() -> Result<()> {
-    let manifest = Manifest::simple(vec![Requirement::from_str("black>21").unwrap()]);
+    let manifest = Manifest::simple(vec![Requirement::from_pep508(
+        pep508_rs::Requirement::from_str("black>21").unwrap(),
+    )
+    .unwrap()]);
     let options = OptionsBuilder::new()
         .resolution_mode(ResolutionMode::Lowest)
         .exclude_newer(Some(*EXCLUDE_NEWER))
@@ -405,7 +442,10 @@ async fn black_lowest() -> Result<()> {
 
 #[tokio::test]
 async fn black_lowest_direct() -> Result<()> {
-    let manifest = Manifest::simple(vec![Requirement::from_str("black>21").unwrap()]);
+    let manifest = Manifest::simple(vec![Requirement::from_pep508(
+        pep508_rs::Requirement::from_str("black>21").unwrap(),
+    )
+    .unwrap()]);
     let options = OptionsBuilder::new()
         .resolution_mode(ResolutionMode::LowestDirect)
         .exclude_newer(Some(*EXCLUDE_NEWER))
@@ -433,12 +473,12 @@ async fn black_lowest_direct() -> Result<()> {
 #[tokio::test]
 async fn black_respect_preference() -> Result<()> {
     let manifest = Manifest::new(
-        vec![Requirement::from_str("black<=23.9.1")?],
+        vec![Requirement::from_pep508(pep508_rs::Requirement::from_str("black<=23.9.1")?).unwrap()],
         Constraints::default(),
         Overrides::default(),
-        vec![Preference::from_requirement(Requirement::from_str(
-            "black==23.9.0",
-        )?)],
+        vec![Preference::from_requirement(
+            Requirement::from_pep508(pep508_rs::Requirement::from_str("black==23.9.0")?).unwrap(),
+        )],
         None,
         vec![],
         Exclusions::default(),
@@ -471,12 +511,12 @@ async fn black_respect_preference() -> Result<()> {
 #[tokio::test]
 async fn black_ignore_preference() -> Result<()> {
     let manifest = Manifest::new(
-        vec![Requirement::from_str("black<=23.9.1")?],
+        vec![Requirement::from_pep508(pep508_rs::Requirement::from_str("black<=23.9.1")?).unwrap()],
         Constraints::default(),
         Overrides::default(),
-        vec![Preference::from_requirement(Requirement::from_str(
-            "black==23.9.2",
-        )?)],
+        vec![Preference::from_requirement(
+            Requirement::from_pep508(pep508_rs::Requirement::from_str("black==23.9.2")?).unwrap(),
+        )],
         None,
         vec![],
         Exclusions::default(),
@@ -507,7 +547,10 @@ async fn black_ignore_preference() -> Result<()> {
 
 #[tokio::test]
 async fn black_disallow_prerelease() -> Result<()> {
-    let manifest = Manifest::simple(vec![Requirement::from_str("black<=20.0").unwrap()]);
+    let manifest = Manifest::simple(vec![Requirement::from_pep508(
+        pep508_rs::Requirement::from_str("black<=20.0").unwrap(),
+    )
+    .unwrap()]);
     let options = OptionsBuilder::new()
         .prerelease_mode(PreReleaseMode::Disallow)
         .exclude_newer(Some(*EXCLUDE_NEWER))
@@ -528,7 +571,10 @@ async fn black_disallow_prerelease() -> Result<()> {
 
 #[tokio::test]
 async fn black_allow_prerelease_if_necessary() -> Result<()> {
-    let manifest = Manifest::simple(vec![Requirement::from_str("black<=20.0").unwrap()]);
+    let manifest = Manifest::simple(vec![Requirement::from_pep508(
+        pep508_rs::Requirement::from_str("black<=20.0").unwrap(),
+    )
+    .unwrap()]);
     let options = OptionsBuilder::new()
         .prerelease_mode(PreReleaseMode::IfNecessary)
         .exclude_newer(Some(*EXCLUDE_NEWER))
@@ -549,7 +595,10 @@ async fn black_allow_prerelease_if_necessary() -> Result<()> {
 
 #[tokio::test]
 async fn pylint_disallow_prerelease() -> Result<()> {
-    let manifest = Manifest::simple(vec![Requirement::from_str("pylint==2.3.0").unwrap()]);
+    let manifest = Manifest::simple(vec![Requirement::from_pep508(
+        pep508_rs::Requirement::from_str("pylint==2.3.0").unwrap(),
+    )
+    .unwrap()]);
     let options = OptionsBuilder::new()
         .prerelease_mode(PreReleaseMode::Disallow)
         .exclude_newer(Some(*EXCLUDE_NEWER))
@@ -572,7 +621,10 @@ async fn pylint_disallow_prerelease() -> Result<()> {
 
 #[tokio::test]
 async fn pylint_allow_prerelease() -> Result<()> {
-    let manifest = Manifest::simple(vec![Requirement::from_str("pylint==2.3.0").unwrap()]);
+    let manifest = Manifest::simple(vec![Requirement::from_pep508(
+        pep508_rs::Requirement::from_str("pylint==2.3.0").unwrap(),
+    )
+    .unwrap()]);
     let options = OptionsBuilder::new()
         .prerelease_mode(PreReleaseMode::Allow)
         .exclude_newer(Some(*EXCLUDE_NEWER))
@@ -596,8 +648,10 @@ async fn pylint_allow_prerelease() -> Result<()> {
 #[tokio::test]
 async fn pylint_allow_explicit_prerelease_without_marker() -> Result<()> {
     let manifest = Manifest::simple(vec![
-        Requirement::from_str("pylint==2.3.0").unwrap(),
-        Requirement::from_str("isort>=5.0.0").unwrap(),
+        Requirement::from_pep508(pep508_rs::Requirement::from_str("pylint==2.3.0").unwrap())
+            .unwrap(),
+        Requirement::from_pep508(pep508_rs::Requirement::from_str("isort>=5.0.0").unwrap())
+            .unwrap(),
     ]);
     let options = OptionsBuilder::new()
         .prerelease_mode(PreReleaseMode::Explicit)
@@ -622,8 +676,10 @@ async fn pylint_allow_explicit_prerelease_without_marker() -> Result<()> {
 #[tokio::test]
 async fn pylint_allow_explicit_prerelease_with_marker() -> Result<()> {
     let manifest = Manifest::simple(vec![
-        Requirement::from_str("pylint==2.3.0").unwrap(),
-        Requirement::from_str("isort>=5.0.0b").unwrap(),
+        Requirement::from_pep508(pep508_rs::Requirement::from_str("pylint==2.3.0").unwrap())
+            .unwrap(),
+        Requirement::from_pep508(pep508_rs::Requirement::from_str("isort>=5.0.0b").unwrap())
+            .unwrap(),
     ]);
     let options = OptionsBuilder::new()
         .prerelease_mode(PreReleaseMode::Explicit)
@@ -649,7 +705,10 @@ async fn pylint_allow_explicit_prerelease_with_marker() -> Result<()> {
 /// fail with a pre-release-centric hint.
 #[tokio::test]
 async fn msgraph_sdk() -> Result<()> {
-    let manifest = Manifest::simple(vec![Requirement::from_str("msgraph-sdk==1.0.0").unwrap()]);
+    let manifest = Manifest::simple(vec![Requirement::from_pep508(
+        pep508_rs::Requirement::from_str("msgraph-sdk==1.0.0").unwrap(),
+    )
+    .unwrap()]);
     let options = OptionsBuilder::new()
         .exclude_newer(Some(*EXCLUDE_NEWER))
         .build();
@@ -669,19 +728,19 @@ async fn msgraph_sdk() -> Result<()> {
 }
 
 static MARKERS_311: Lazy<MarkerEnvironment> = Lazy::new(|| {
-    MarkerEnvironment {
-        implementation_name: "cpython".to_string(),
-        implementation_version: StringVersion::from_str("3.11.5").unwrap(),
-        os_name: "posix".to_string(),
-        platform_machine: "arm64".to_string(),
-        platform_python_implementation: "CPython".to_string(),
-        platform_release: "21.6.0".to_string(),
-        platform_system: "Darwin".to_string(),
-        platform_version: "Darwin Kernel Version 21.6.0: Mon Aug 22 20:19:52 PDT 2022; root:xnu-8020.140.49~2/RELEASE_ARM64_T6000".to_string(),
-        python_full_version: StringVersion::from_str("3.11.5").unwrap(),
-        python_version: StringVersion::from_str("3.11").unwrap(),
-        sys_platform: "darwin".to_string(),
-    }
+    MarkerEnvironment::try_from(MarkerEnvironmentBuilder {
+        implementation_name: "cpython",
+        implementation_version: "3.11.5",
+        os_name: "posix",
+        platform_machine: "arm64",
+        platform_python_implementation: "CPython",
+        platform_release: "21.6.0",
+        platform_system: "Darwin",
+        platform_version: "Darwin Kernel Version 21.6.0: Mon Aug 22 20:19:52 PDT 2022; root:xnu-8020.140.49~2/RELEASE_ARM64_T6000",
+        python_full_version: "3.11.5",
+        python_version: "3.11",
+        sys_platform: "darwin",
+    }).unwrap()
 });
 
 static TAGS_311: Lazy<Tags> = Lazy::new(|| {
@@ -696,24 +755,25 @@ static TAGS_311: Lazy<Tags> = Lazy::new(|| {
         (3, 11),
         "cpython",
         (3, 11),
+        false,
     )
     .unwrap()
 });
 
 static MARKERS_310: Lazy<MarkerEnvironment> = Lazy::new(|| {
-    MarkerEnvironment {
-        implementation_name: "cpython".to_string(),
-        implementation_version: StringVersion::from_str("3.10.5").unwrap(),
-        os_name: "posix".to_string(),
-        platform_machine: "arm64".to_string(),
-        platform_python_implementation: "CPython".to_string(),
-        platform_release: "21.6.0".to_string(),
-        platform_system: "Darwin".to_string(),
-        platform_version: "Darwin Kernel Version 21.6.0: Mon Aug 22 20:19:52 PDT 2022; root:xnu-8020.140.49~2/RELEASE_ARM64_T6000".to_string(),
-        python_full_version: StringVersion::from_str("3.10.5").unwrap(),
-        python_version: StringVersion::from_str("3.10").unwrap(),
-        sys_platform: "darwin".to_string(),
-    }
+    MarkerEnvironment::try_from(MarkerEnvironmentBuilder {
+        implementation_name: "cpython",
+        implementation_version: "3.10.5",
+        os_name: "posix",
+        platform_machine: "arm64",
+        platform_python_implementation: "CPython",
+        platform_release: "21.6.0",
+        platform_system: "Darwin",
+        platform_version: "Darwin Kernel Version 21.6.0: Mon Aug 22 20:19:52 PDT 2022; root:xnu-8020.140.49~2/RELEASE_ARM64_T6000",
+        python_full_version: "3.10.5",
+        python_version: "3.10",
+        sys_platform: "darwin",
+    }).unwrap()
 });
 
 static TAGS_310: Lazy<Tags> = Lazy::new(|| {
@@ -728,6 +788,7 @@ static TAGS_310: Lazy<Tags> = Lazy::new(|| {
         (3, 10),
         "cpython",
         (3, 10),
+        false,
     )
     .unwrap()
 });
