@@ -1,10 +1,12 @@
 //! Fundamental types shared across `uv` crates.
 pub use builds::*;
 pub use downloads::*;
+pub use hash::*;
 pub use requirements::*;
 pub use traits::*;
 
 mod builds;
 mod downloads;
+mod hash;
 mod requirements;
 mod traits;
