@@ -1,5 +1,7 @@
+pub use crate::combine::*;
 pub use crate::settings::*;
 pub use crate::workspace::*;
 
+mod combine;
 mod settings;
 mod workspace;
