@@ -11,8 +11,8 @@ use owo_colors::OwoColorize;
 use tracing::instrument;
 
 use uv_cache::Cache;
-
 use uv_requirements::RequirementsSource;
+use uv_workspace::Combine;
 
 use crate::cli::{CacheCommand, CacheNamespace, Cli, Commands, PipCommand, PipNamespace};
 #[cfg(feature = "self-update")]
@@ -112,10 +112,12 @@ async fn run() -> Result<ExitStatus> {
     // 3. The user configuration file.
     let workspace = if let Some(config_file) = cli.config_file.as_ref() {
         Some(uv_workspace::Workspace::from_file(config_file)?)
-    } else if let Some(workspace) = uv_workspace::Workspace::find(env::current_dir()?)? {
-        Some(workspace)
+    } else if cli.global_args.isolated {
+        None
     } else {
-        uv_workspace::Workspace::user()?
+        let project = uv_workspace::Workspace::find(env::current_dir()?)?;
+        let user = uv_workspace::Workspace::user()?;
+        project.combine(user)
     };
 
     // Resolve the global settings.
@@ -227,8 +229,11 @@ async fn run() -> Result<ExitStatus> {
                 args.shared.link_mode,
                 args.shared.python,
                 args.shared.system,
+                args.shared.concurrency,
+                args.uv_lock,
                 globals.native_tls,
                 globals.quiet,
+                globals.preview,
                 cache,
                 printer,
             )
@@ -270,7 +275,10 @@ async fn run() -> Result<ExitStatus> {
                 args.shared.python,
                 args.shared.system,
                 args.shared.break_system_packages,
+                args.shared.target,
+                args.shared.concurrency,
                 globals.native_tls,
+                globals.preview,
                 cache,
                 printer,
             )
@@ -279,6 +287,7 @@ async fn run() -> Result<ExitStatus> {
         Commands::Pip(PipNamespace {
             command: PipCommand::Install(args),
         }) => {
+            args.compat_args.validate()?;
             // Resolve the settings from the command-line arguments and workspace configuration.
             let args = PipInstallSettings::resolve(args, workspace);
 
@@ -334,7 +343,11 @@ async fn run() -> Result<ExitStatus> {
                 args.shared.python,
                 args.shared.system,
                 args.shared.break_system_packages,
+                args.shared.target,
+                args.shared.concurrency,
+                args.uv_lock,
                 globals.native_tls,
+                globals.preview,
                 cache,
                 args.dry_run,
                 printer,
@@ -362,9 +375,11 @@ async fn run() -> Result<ExitStatus> {
                 args.shared.python,
                 args.shared.system,
                 args.shared.break_system_packages,
+                args.shared.target,
                 cache,
                 args.shared.connectivity,
                 globals.native_tls,
+                globals.preview,
                 args.shared.keyring_provider,
                 printer,
             )
@@ -472,6 +487,7 @@ async fn run() -> Result<ExitStatus> {
                 args.system_site_packages,
                 args.shared.connectivity,
                 args.seed,
+                args.allow_existing,
                 args.shared.exclude_newer,
                 globals.native_tls,
                 &cache,
@@ -508,14 +524,25 @@ async fn run() -> Result<ExitStatus> {
                 args.args,
                 requirements,
                 args.python,
-                args.isolated,
-                args.no_workspace,
+                globals.isolated,
                 globals.preview,
                 &cache,
                 printer,
             )
             .await
         }
+        Commands::Sync(args) => {
+            // Resolve the settings from the command-line arguments and workspace configuration.
+            let _args = settings::SyncSettings::resolve(args, workspace);
+
+            commands::sync(globals.preview, &cache, printer).await
+        }
+        Commands::Lock(args) => {
+            // Resolve the settings from the command-line arguments and workspace configuration.
+            let _args = settings::LockSettings::resolve(args, workspace);
+
+            commands::lock(globals.preview, &cache, printer).await
+        }
         #[cfg(feature = "self-update")]
         Commands::Self_(SelfNamespace {
             command: SelfCommand::Update,
